@@ -1,3 +1,4 @@
+/* -*- mode: c; c-style: k&r; c-basic-offset: 8 -*- */
 /*
  *  Copyright (C) 2002  Ricardo Fernndez Pascual
  *
@@ -23,7 +24,6 @@
 #include "bookmarks-context-menu.h"
 #include <glib/gi18n.h>
 #include "galeon-marshal.h"
-#include "gul-gobject-misc.h"
 #include "gul-gui.h"
 #include "bookmarks-single-editor.h"
 #include "bookmarks-clipboard.h"
@@ -36,6 +36,10 @@
 /**
  * Private data
  */
+#define GB_CONTEXT_MENU_GET_PRIVATE(object) (G_TYPE_INSTANCE_GET_PRIVATE ((object), \
+				       GB_TYPE_CONTEXT_MENU, GbContextMenuPrivate))
+
+
 struct _GbContextMenuPrivate 
 {
 	GbBookmark *bm;
@@ -46,8 +50,6 @@ struct _GbContextMenuPrivate
 /**
  * Private functions, only availble from this file
  */
-static void		gb_context_menu_class_init		(GbContextMenuClass *klass);
-static void		gb_context_menu_init			(GbContextMenu *cm);
 static void		gb_context_menu_build_menu		(GbContextMenu *cm);
 static void		gb_context_menu_finalize_impl		(GObject *o);
 
@@ -73,7 +75,6 @@ static void		gb_context_menu_open_folder_windows_cb	(GtkWidget *menu_item, GbCon
 static void		gb_context_menu_show_as_toolbar_cb	(GtkWidget *menu_item, GbContextMenu *cm);
 static void		gb_context_menu_set_as_default_folder_cb (GtkWidget *menu_item, GbContextMenu *cm);
 
-static gpointer g_object_class;
 
 enum GbContextMenuSignalsEnum {
 	GB_CONTEXT_MENU_BOOKMARK_ACTIVATED,
@@ -86,14 +87,12 @@ static gint GbContextMenuSignals[GB_CONTEXT_MENU_LAST_SIGNAL];
  * ContextMenu object
  */
 
-MAKE_GET_TYPE (gb_context_menu, "GbContextMenu", GbContextMenu, gb_context_menu_class_init, 
-	       gb_context_menu_init, G_TYPE_OBJECT);
+G_DEFINE_TYPE (GbContextMenu, gb_context_menu, G_TYPE_OBJECT);
 
 static void
 gb_context_menu_class_init (GbContextMenuClass *klass)
 {
 	G_OBJECT_CLASS (klass)->finalize = gb_context_menu_finalize_impl;
-	g_object_class = g_type_class_peek_parent (klass);
 
 	GbContextMenuSignals[GB_CONTEXT_MENU_BOOKMARK_ACTIVATED] = g_signal_new (
 		"bookmark-activated", G_OBJECT_CLASS_TYPE (klass),  
@@ -109,12 +108,14 @@ gb_context_menu_class_init (GbContextMenuClass *klass)
 		NULL, NULL, 
 		galeon_marshal_VOID__VOID,
 		G_TYPE_NONE, 0);
+
+	g_type_class_add_private (klass, sizeof (GbContextMenuPrivate));
 }
 
 static void 
 gb_context_menu_init (GbContextMenu *cm)
 {
-	GbContextMenuPrivate *p = g_new0 (GbContextMenuPrivate, 1);
+	GbContextMenuPrivate *p = GB_CONTEXT_MENU_GET_PRIVATE (cm);
 	cm->priv = p;
 }
 
@@ -138,9 +139,8 @@ gb_context_menu_finalize_impl (GObject *o)
 		g_object_unref (p->menu);
 	}
 	
-	g_free (p);
 	
-	G_OBJECT_CLASS (g_object_class)->finalize (o);
+	G_OBJECT_CLASS (gb_context_menu_parent_class)->finalize (o);
 
 }
 
@@ -206,6 +206,18 @@ gb_context_menu_gb_bookmark_alias_make_real_cb (GtkWidget *menu_item, GbContextM
 	g_object_unref (cm);
 }
 #endif 
+#if 0
+static void
+gb_context_menu_gb_autofolder_refresh_cb (GtkWidget *menu_item, GbContextMenu *cm)
+{
+	GbContextMenuPrivate *p = cm->priv;
+	GbBookmark *bm = p->bm;
+
+	gb_auto_folder_refresh (GB_AUTO_FOLDER (bm));
+
+	g_object_unref (cm);
+}
+#endif
 
 static void
 gb_context_menu_build_menu (GbContextMenu *cm)
@@ -228,12 +240,12 @@ gb_context_menu_build_menu (GbContextMenu *cm)
 
 	if (GB_IS_SITE (p->bm))
 	{
+		gul_gui_append_new_menuitem_stock_icon
+			(p->menu,  STOCK_NEW_TAB, _("Open in New _Tab"),
+			 G_CALLBACK (gb_context_menu_open_new_tab_cb), cm);
 		gul_gui_append_new_menuitem_stock_icon 
 			(p->menu, GTK_STOCK_OPEN, _("Open in New _Window"),
 			 G_CALLBACK (gb_context_menu_open_new_window_cb), cm);
-		gul_gui_append_new_menuitem
-			(p->menu,  _("Open in New _Tab"),
-			 G_CALLBACK (gb_context_menu_open_new_tab_cb), cm);
 		gul_gui_append_new_menuitem_stock_icon 
 			(p->menu, GTK_STOCK_COPY, _("Copy _Link Location"),
 			 G_CALLBACK (gb_context_menu_copy_location_cb), cm);
@@ -243,29 +255,37 @@ gb_context_menu_build_menu (GbContextMenu *cm)
 	{
 		if (p->location_source)
 		{
-			gul_gui_append_new_menuitem_stock_icon 
+			item = gul_gui_append_new_menuitem_stock_icon 
 				(p->menu, STOCK_ADD_BOOKMARK, _("Add _Bookmark Here"),
 				 G_CALLBACK (gb_context_menu_add_bookmark_here_cb), cm);
 		}
 		else
 		{
-			gul_gui_append_new_menuitem_stock_icon
+			item = gul_gui_append_new_menuitem_stock_icon
 				(p->menu, GTK_STOCK_NEW, _("New _Bookmark"),
 				 G_CALLBACK (gb_context_menu_new_bookmark_cb), cm);
 		}
+		if (gb_folder_is_autogenerated (GB_FOLDER (p->bm)))
+		{
+			gtk_widget_set_sensitive (item, FALSE);
+		}
 
-		gul_gui_append_new_menuitem
+		item = gul_gui_append_new_menuitem
 			(p->menu, _("New _Folder"),
 			 G_CALLBACK (gb_context_menu_add_folder_here_cb), cm);
+		if (gb_folder_is_autogenerated (GB_FOLDER (p->bm)))
+		{
+			gtk_widget_set_sensitive (item, FALSE);
+		}
 
 		gul_gui_append_separator (p->menu);
 
-		gul_gui_append_new_menuitem_stock_icon 
-			(p->menu, GTK_STOCK_OPEN, _("Open Folder in _Windows"),
-			 G_CALLBACK (gb_context_menu_open_folder_windows_cb), cm);
 		gul_gui_append_new_menuitem
 			(p->menu, _("Open Folder in _Tabs"),
 			 G_CALLBACK (gb_context_menu_open_folder_tabs_cb), cm);
+		gul_gui_append_new_menuitem_stock_icon 
+			(p->menu, GTK_STOCK_OPEN, _("Open Folder in _Windows"),
+			 G_CALLBACK (gb_context_menu_open_folder_windows_cb), cm);
 /*
 		gul_gui_append_new_menuitem_stock_icon 
 			(p->menu, GTK_STOCK_OPEN, _("Open folder in _My Portal"),
@@ -281,35 +301,49 @@ gb_context_menu_build_menu (GbContextMenu *cm)
 		item = gul_gui_append_new_menuitem
 			(p->menu, _("_Set as Default Folder"), 
 			 G_CALLBACK (gb_context_menu_set_as_default_folder_cb), cm);
-               if (gb_folder_is_default_folder (GB_FOLDER (p->bm)))
-	       {
-                       gtk_widget_set_sensitive (item, FALSE);
-               }
-
+		if (gb_folder_is_default_folder (GB_FOLDER (p->bm))
+		    || gb_folder_is_autogenerated (GB_FOLDER (p->bm)))
+		{
+			gtk_widget_set_sensitive (item, FALSE);
+		}
+		
 	}
 
 	gul_gui_append_separator (p->menu);
-
-	gul_gui_append_new_menuitem_stock
+	
+	item = gul_gui_append_new_menuitem_stock
 		(p->menu, GTK_STOCK_CUT,
 		 G_CALLBACK (gb_context_menu_cut_cb), cm);
-
+	if (!p->bm->parent || gb_folder_is_autogenerated (p->bm->parent))
+	{
+		gtk_widget_set_sensitive (item, FALSE);
+	}
+	
 	gul_gui_append_new_menuitem_stock
 		(p->menu, GTK_STOCK_COPY,
 		 G_CALLBACK (gb_context_menu_copy_cb), cm);
-
-	gul_gui_append_new_menuitem_stock
+	
+	item = gul_gui_append_new_menuitem_stock
 		(p->menu, GTK_STOCK_PASTE,
 		 G_CALLBACK (gb_context_menu_paste_cb), cm);
+	if ((GB_IS_FOLDER (p->bm) && gb_folder_is_autogenerated (GB_FOLDER (p->bm)))
+	    || (!p->bm->parent || gb_folder_is_autogenerated (p->bm->parent)))
+	{
+		gtk_widget_set_sensitive (item, FALSE);
+	}
 	
 	gul_gui_append_separator (p->menu);
 
-	gul_gui_append_new_menuitem_stock
-			(p->menu, GTK_STOCK_DELETE, 
-			 G_CALLBACK (gb_context_menu_remove_cb), cm);
-			
+	item = gul_gui_append_new_menuitem_stock
+		(p->menu, GTK_STOCK_DELETE, 
+		 G_CALLBACK (gb_context_menu_remove_cb), cm);
+	if (!p->bm->parent || gb_folder_is_autogenerated (p->bm->parent))
+	{
+		gtk_widget_set_sensitive (item, FALSE);
+	}
+	
 	gul_gui_append_separator (p->menu);
-
+	
 	gul_gui_append_new_menuitem_stock 
 		(p->menu, GTK_STOCK_PROPERTIES,
 		 G_CALLBACK (gb_context_menu_edit_cb), cm);
@@ -320,6 +354,16 @@ gb_context_menu_build_menu (GbContextMenu *cm)
 		(p->menu, "gb_bookmark_alias_make_real",
 		 G_CALLBACK (gb_context_menu_gb_bookmark_alias_make_real_cb), cm);
 #endif
+#if 0
+	/* to debug auto folders */
+	if (GB_IS_AUTO_FOLDER (p->bm))
+	{
+		gul_gui_append_new_menuitem
+			(p->menu, "gb_auto_folder_refresh",
+			 G_CALLBACK (gb_context_menu_gb_autofolder_refresh_cb), cm);
+	}
+	
+#endif
 }
 
 /**
@@ -433,6 +477,7 @@ gb_context_menu_paste_cb (GtkWidget *menu_item, GbContextMenu *cm)
 	gint idx;
 
 	g_return_if_fail (GB_IS_FOLDER (f));
+	g_return_if_fail (!gb_folder_is_autogenerated (f));
 
 	idx = GB_IS_FOLDER (bm) ? 0 : gb_folder_get_child_index (bm->parent, bm);
 
