@@ -37,6 +37,7 @@
 #include <gtk/gtkmessagedialog.h>
 #include <gtk/gtkprogressbar.h>
 #include <gtk/gtkexpander.h>
+#include <gtk/gtkmain.h>
 
 #include <glib/gi18n.h>
 #include <libgnomeui/gnome-dialog-util.h>
@@ -60,14 +61,21 @@ enum
 	COL_FILENAME,
 	COL_SIZE,
 	COL_REMAINING,
-	COL_DETAILS
+	COL_DETAILS,
+	COL_REMAINING_TIME
 };
 
+#define DOWNLOADER_VIEW_GET_PRIVATE(object) (G_TYPE_INSTANCE_GET_PRIVATE ((object), \
+				       TYPE_DOWNLOADER_VIEW, DownloaderViewPrivate))
+
+
 struct DownloaderViewPrivate
 {
 	GtkTreeModel *model;
 	gboolean show_details;
 
+	gboolean deleting;
+
 	/* Widgets */
 	GtkWidget *treeview;
 	GtkWidget *details_file;
@@ -84,7 +92,7 @@ struct DownloaderViewPrivate
 	GtkWidget *open_button;
 };
 
-#define DOWNLOAD_TYPE	(download_get_type ())
+#define TYPE_DOWNLOAD	(download_get_type ())
 
 typedef enum
 {
@@ -109,9 +117,9 @@ struct _Download
 
 	guint  elapsed;		/* seconds */
 	guint  remaining;	/* seconds (G_MAXUINT if indeterminate) */
-	guint  speed;		/* bytes per second */
-	gulong size_total;	/* bytes (G_MAXULONG if indeterminate) */
-	gulong size_done;	/* bytes */
+	guint64  speed;		/* bytes per second */
+	guint64 size_total;	/* bytes (G_MAXUINT64 if indeterminate) */
+	guint64 size_done;	/* bytes */
 	gfloat progress;	/* 0.0 .. 1.0 (or < 0 if indeterminate) */
 
 	gboolean can_pause;
@@ -232,7 +240,8 @@ enum
 	PROP_RESUME_BUTTON,
 	PROP_ABORT_BUTTON,
 	PROP_OPEN_BUTTON,
-	PROP_DETAILS_BUTTON
+	PROP_DETAILS_BUTTON,
+	PROP_DOWNLOAD_VBOX
 };
 
 static const
@@ -252,6 +261,7 @@ GaleonDialogProperty properties [] =
 	{ PROP_ABORT_BUTTON, "abort_button", NULL, PT_NORMAL, NULL },
 	{ PROP_OPEN_BUTTON, "open_button", NULL, PT_NORMAL, NULL },
 	{ PROP_DETAILS_BUTTON, "details_expander", CONF_DOWNLOADING_SHOW_DETAILS, PT_NORMAL, NULL },
+	{ PROP_DOWNLOAD_VBOX, "download_vbox", NULL, PT_NORMAL, NULL },
 	
         { -1, NULL, NULL }
 };
@@ -308,7 +318,7 @@ downloader_view_get_type (void)
                         (GInstanceInitFunc) downloader_view_init
                 };
 
-                downloader_view_type = g_type_register_static (GALEON_DIALOG_TYPE,
+                downloader_view_type = g_type_register_static (GALEON_TYPE_DIALOG,
                                                                "DownloaderView",
                                                                &our_info, 0);
         }
@@ -347,6 +357,8 @@ downloader_view_class_init (DownloaderViewClass *klass)
         galeon_dialog_class = GALEON_DIALOG_CLASS (klass);
 
         object_class->finalize = downloader_view_finalize;
+
+	g_type_class_add_private (klass, sizeof (DownloaderViewPrivate));
 }
 
 static gboolean
@@ -362,7 +374,9 @@ destroy_details_foreach (GtkTreeModel *model,
 static void
 downloader_view_init (DownloaderView *dv)
 {
-        dv->priv = g_new0 (DownloaderViewPrivate, 1);
+        dv->priv = DOWNLOADER_VIEW_GET_PRIVATE (dv);
+
+	dv->priv->deleting = FALSE;
 	
 	downloader_view_build_ui (dv);
 
@@ -385,7 +399,7 @@ downloader_view_finalize (GObject *object)
 
 	gtk_tree_model_foreach (dv->priv->model, destroy_details_foreach, NULL);
 
-	g_free (dv->priv);
+	g_object_unref (dv->priv->model);
 
         G_OBJECT_CLASS (parent_class)->finalize (object);
 
@@ -397,7 +411,7 @@ DownloaderView *
 downloader_view_new (void)
 {
 	return DOWNLOADER_VIEW (g_object_new 
-			(DOWNLOADER_VIEW_TYPE, NULL));
+			(TYPE_DOWNLOADER_VIEW, NULL));
 }
 
 static void
@@ -620,7 +634,7 @@ downloader_view_set_download_info (DownloaderViewPrivate *priv,
 			    -1);
 	
 	/* Total */
-	if (details->size_total < G_MAXULONG)
+	if (details->size_total < G_MAXUINT64)
 	{
 		size = gnome_vfs_format_file_size_for_display (details->size_total);
 	}
@@ -646,6 +660,7 @@ downloader_view_set_download_info (DownloaderViewPrivate *priv,
 	gtk_list_store_set (GTK_LIST_STORE (priv->model),
 			    iter,
 			    COL_REMAINING, buffer,
+			    COL_REMAINING_TIME, details->remaining,
 			    -1);
 
 	if (gtk_tree_path_compare (path, selected_path) == 0)
@@ -688,7 +703,7 @@ downloader_view_add_download (DownloaderView *dv,
 	DownloadDetails *details;
 	GtkTreeSelection *selection;
 	
-	details = g_object_new (DOWNLOAD_TYPE, NULL);
+	details = g_object_new (TYPE_DOWNLOAD, NULL);
 	details->view = dv;
 	details->filename = g_path_get_basename (dest);
 	details->source = g_strdup (source);
@@ -696,7 +711,7 @@ downloader_view_add_download (DownloaderView *dv,
 	details->elapsed = 0;
 	details->remaining = G_MAXUINT;
 	details->speed = 0;
-	details->size_total = G_MAXULONG;
+	details->size_total = G_MAXUINT64;
 	details->size_done = 0;
 	details->progress = -1;
 	dv->priv->show_details = FALSE;
@@ -742,7 +757,15 @@ downloader_view_remove_download (DownloaderView *dv,
 				 &iter, path);
 	gtk_tree_path_free (path);
 
-	gtk_list_store_remove (GTK_LIST_STORE (dv->priv->model), &iter);
+	/*
+	 * If we're deleting the dialog, then we don't need to remove the
+	 * row, and we'll actually cause problems if we do because this
+	 * call is made during a foreach iteration over the store.
+	 */
+	if (!dv->priv->deleting)
+	{
+		gtk_list_store_remove (GTK_LIST_STORE (dv->priv->model), &iter);
+	}
 
         galeon_dialog_get_value (GALEON_DIALOG(dv), PROP_KEEP_OPEN, &keep_open);
 	
@@ -761,9 +784,9 @@ void
 download_set_progress (Download	*details,
 		       guint	 elapsed,
 		       guint	 remaining,
-		       guint	 speed,
-		       gulong	 size_done,
-		       gulong	 size_total)
+		       guint64	 speed,
+		       guint64	 size_done,
+		       guint64	 size_total)
 {
 	details->elapsed = elapsed;
 	details->remaining = remaining;
@@ -779,14 +802,14 @@ download_set_progress (Download	*details,
 }
 
 void
-download_set_size_total (DownloadDetails *details, gulong size_total)
+download_set_size_total (DownloadDetails *details, guint64 size_total)
 {
 	DownloaderView *dv;
 	GtkTreePath *path;
 	GtkTreeIter  iter;
 
 	details->size_total = size_total;
-	if (size_total > 0 && size_total < G_MAXULONG)
+	if (size_total > 0 && size_total < G_MAXUINT64)
 	{
 		details->progress = (gfloat)details->size_done / size_total;
 	}
@@ -853,7 +876,7 @@ download_resumed (Download *self)
 void
 download_completed (Download *self)
 {
-	if (self->size_total < G_MAXULONG)
+	if (self->size_total < G_MAXUINT64)
 	{
 		self->size_done = self->size_total;
 	}
@@ -883,6 +906,7 @@ downloader_view_build_ui (DownloaderView *dv)
 	GtkCellRenderer *renderer;
 	GtkTreeSelection *selection;
 	GtkWidget *details_table;
+	GtkWidget *download_vbox;
 	GaleonDialog *d = GALEON_DIALOG (dv);
 
 	galeon_dialog_construct (d,
@@ -904,6 +928,10 @@ downloader_view_build_ui (DownloaderView *dv)
 	priv->open_button = galeon_dialog_get_control (d, PROP_OPEN_BUTTON);
 	details_table = galeon_dialog_get_control (d, PROP_DETAILS_TABLE);
 
+	/* See bug 139561 */
+	download_vbox = galeon_dialog_get_control (d, PROP_DOWNLOAD_VBOX);
+	gtk_widget_set_redraw_on_allocate (GTK_WIDGET (download_vbox), TRUE);
+
 	/* Set inital visibility state of details */
 	download_dialog_details_cb (GTK_EXPANDER (priv->details_button), NULL, dv);
 
@@ -920,12 +948,13 @@ downloader_view_build_ui (DownloaderView *dv)
 	gtk_widget_show (priv->details_location);
 	gtk_widget_show (priv->details_file);
 
-	liststore = gtk_list_store_new (5,
+	liststore = gtk_list_store_new (6,
 					G_TYPE_STRING,
 					G_TYPE_STRING,
 					G_TYPE_STRING,
 					G_TYPE_STRING,
-					DOWNLOAD_TYPE);
+					TYPE_DOWNLOAD,
+					G_TYPE_INT);
 
 	gtk_tree_view_set_model (GTK_TREE_VIEW(priv->treeview), 
 				 GTK_TREE_MODEL (liststore));
@@ -975,7 +1004,7 @@ downloader_view_build_ui (DownloaderView *dv)
 					   COL_REMAINING);
         gtk_tree_view_column_set_resizable (column, TRUE);
         gtk_tree_view_column_set_reorderable (column, TRUE);
-        gtk_tree_view_column_set_sort_column_id (column, COL_REMAINING);
+        gtk_tree_view_column_set_sort_column_id (column, COL_REMAINING_TIME);
 
 	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW(priv->treeview));
 	gtk_tree_selection_set_mode (selection, GTK_SELECTION_MULTIPLE);
@@ -1140,7 +1169,7 @@ delete_pending_foreach (GtkTreeModel *model,
 			gpointer      user_data)
 {
 	DownloadDetails *details;
-	
+
 	gtk_tree_model_get (model, iter, COL_DETAILS, &details, -1);
 	g_return_val_if_fail (details != NULL, TRUE);
 
@@ -1178,7 +1207,9 @@ download_dialog_delete_cb (GtkWidget *window, GdkEventAny *event,
 	/* do the appropriate thing */
 	if (choice == GTK_RESPONSE_YES)
 	{
+		dv->priv->deleting = TRUE;
 		gtk_tree_model_foreach (dv->priv->model, delete_pending_foreach, NULL);
+		dv->priv->deleting = FALSE;
 		return FALSE;
 	}
 
@@ -1239,9 +1270,11 @@ open_selection_foreach (GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *ite
 	app = gnome_vfs_mime_get_default_application (mime);
 	if (app)
 	{            
-		gul_general_launch_application (app->command,
-						details->dest,
-						app->requires_terminal);
+		/* FIXME: check return value */
+		/* Current event time is fine here as it was called via a user
+		 * button click time */
+		gul_general_launch_application (app, details->dest,
+						gtk_get_current_event_time());
 	}
 	else
 	{
