@@ -20,24 +20,21 @@
 #include "eel-gconf-extensions.h"
 #include "gul-string.h"
 #include "gul-general.h"
-#include "gul-gobject-misc.h"
 #include "galeon-config.h"
 #include "galeon-autocompletion-source.h"
+#include "galeon-auto-bookmarks-source.h"
+#include "galeon-debug.h"
 
-#include <string.h>
-#include <time.h>
-#include <unistd.h>
-#include <libxml/parser.h>
-#include <libxml/tree.h>
-#include <libxml/xmlmemory.h>
-#include <libgnome/gnome-i18n.h>
 #include <libgnomevfs/gnome-vfs-uri.h>
 #include <libgnome/gnome-util.h>
-#include <fcntl.h>
+#include <libxml/parser.h>
+#include <libxml/xmlwriter.h>
 #include <glib/gstrfuncs.h>
+#include <glib/gi18n.h>
 
-//#define DEBUG_MSG(x) g_print x
-#define DEBUG_MSG(x)
+#include <time.h>
+#include <math.h>
+#include <unistd.h>
 
 /* when parsing using SAX, elements 0 and 1 of the attribute array are
  * the key and value respectively */
@@ -47,6 +44,10 @@ enum
 	SAX_VALUE = 1
 };
 
+#define GLOBAL_HISTORY_GET_PRIVATE(object) (G_TYPE_INSTANCE_GET_PRIVATE ((object), \
+				       TYPE_GLOBAL_HISTORY, GlobalHistoryPrivate))
+
+
 struct GlobalHistoryPrivate
 {
 	GHashTable *history_sites;
@@ -54,6 +55,7 @@ struct GlobalHistoryPrivate
 	HistoryItem *last_visited_site;
 	gboolean history_dirty;
 	gint expire_days;
+	guint timeout;
 };
 
 struct HistoryItemPrivate
@@ -62,6 +64,7 @@ struct HistoryItemPrivate
 	GHashTable *properties;
 	GSList *children;
 	gboolean is_host;
+	gint zoom;
 };
 
 enum
@@ -72,12 +75,10 @@ enum
         LAST_SIGNAL
 };
 
-static void
-global_history_class_init (GlobalHistoryClass *klass);
 static void
 global_history_autocompletion_source_init (GaleonAutocompletionSourceIface *iface);
 static void
-global_history_init (GlobalHistory *gh);
+global_history_auto_bookmarks_source_init (GaleonAutoBookmarksSourceIface *iface);
 static void
 global_history_finalize (GObject *object);
 static void
@@ -89,8 +90,17 @@ static void
 global_history_autocompletion_source_set_basic_key (GaleonAutocompletionSource *source,
 						    const gchar *basic_key);
 static void
+global_history_auto_bookmarks_source_get_autobookmarks	(GaleonAutoBookmarksSource *source, 
+							 GaleonAutoBookmarksSourceIteratorFunc iterator,
+							 gpointer iterator_data,
+							 GaleonAutoBookmarksSourceFilterFunc filter,
+							 gpointer filter_data,
+							 GaleonAutoBookmarksScoringMethod scoring,
+							 gboolean group_by_host);
+static void
 global_history_emit_changed_data_changed (GlobalHistory *gh);
 
+
 static HistoryItem *
 history_add_host (GlobalHistory *gh, 
 		  const char *name);
@@ -104,17 +114,19 @@ history_add_item (GlobalHistory *gh,
 static HistoryItem *
 history_get_host (GlobalHistory *gh, 
 		  const char *url);
-
-static GObjectClass *parent_class = NULL;
+static void
+history_item_free (HistoryItem *hi);
 
 /* how often to save the history, in milliseconds */
 #define HISTORY_SAVE_INTERVAL (60 * 5 * 1000)
 
 static guint global_history_signals[LAST_SIGNAL] = { 0 };
 
-MAKE_GET_TYPE_IFACE (global_history, "GlobalHistory", GlobalHistory,
-		     global_history_class_init, global_history_init, G_TYPE_OBJECT,
-		     global_history_autocompletion_source_init, GALEON_TYPE_AUTOCOMPLETION_SOURCE);
+G_DEFINE_TYPE_WITH_CODE (GlobalHistory, global_history, G_TYPE_OBJECT,
+			 G_IMPLEMENT_INTERFACE (GALEON_TYPE_AUTOCOMPLETION_SOURCE,
+						global_history_autocompletion_source_init)
+			 G_IMPLEMENT_INTERFACE (GALEON_TYPE_AUTO_BOOKMARKS_SOURCE,
+						global_history_auto_bookmarks_source_init));
 
 static void
 global_history_autocompletion_source_init (GaleonAutocompletionSourceIface *iface)
@@ -123,13 +135,17 @@ global_history_autocompletion_source_init (GaleonAutocompletionSourceIface *ifac
 	iface->set_basic_key = global_history_autocompletion_source_set_basic_key;
 }
 
+static void
+global_history_auto_bookmarks_source_init (GaleonAutoBookmarksSourceIface *iface)
+{
+	iface->get_autobookmarks = global_history_auto_bookmarks_source_get_autobookmarks;
+}
+
 static void
 global_history_class_init (GlobalHistoryClass *klass)
 {
         GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
-        parent_class = g_type_class_peek_parent (klass);
-
         object_class->finalize = global_history_finalize;
 
 	global_history_signals[ADD] =
@@ -162,78 +178,10 @@ global_history_class_init (GlobalHistoryClass *klass)
                               G_TYPE_NONE,
                               1,
 			      G_TYPE_POINTER);
+
+	g_type_class_add_private (klass, sizeof (GlobalHistoryPrivate));
 }
 
-/**
- * xml_decode_entity: translate &amp; et al and return a new
- * expanded string 
- */
-static char *
-xml_decode_entity (const unsigned char *encoded)
-{
-	gchar *buffer;
-	gint i, j, length;
-	
-	if (encoded == NULL)
-	{
-		return NULL;
-	}
-
-	buffer = g_strdup (encoded);
-
-	/* Optimize for case where no escape codes are found, otherwise
-	 * use the first found code as the starting index */
-	if ((i = (char *) strstr (buffer, "&") - buffer) < 0)
-	{
-		return buffer;
-	}
-	
-	length = strlen (buffer);
-	for (j = i; i < length; i++)
-	{
-		if (encoded[i] == '&')
-		{
-			if (strncmp (encoded + i + 1, "amp;", 4) == 0)
-			{
-				buffer[j++] = '&';
-				i += 4;
-			}
-			else if (strncmp (encoded + i + 1, "lt;", 3) == 0)
-			{
-				buffer[j++] = '<';
-				i += 3;
-			}
-			else if (strncmp (encoded + i + 1, "gt;", 3) == 0)
-			{
-				buffer[j++] = '>';
-				i += 3;
-			}
-			else if (strncmp (encoded + i + 1, "quot;", 5) == 0)
-			{
-				buffer[j++] = '"';
-				i += 5;
-			}
-			else if (strncmp (encoded + i + 1, "apos;", 5) == 0)
-			{
-				buffer[j++] = '\'';
-				i += 5;
-			}
-			else if (encoded[i + 1] == '#')
-			{
-				buffer[j++] = atoi (encoded + i + 2);
-				i += 5;
-			}
-		}
-		else
-		{
-			buffer[j++] = encoded[i];
-		}
-	}
-
-	buffer[j] = '\0';
-	return buffer;
-}
-				
 /** 
  * Returns the number of days since the history item was visited for the last time
  */
@@ -255,18 +203,23 @@ history_parse_host_element (GlobalHistory *gh, const xmlChar **attrs)
 	const xmlChar **attr;
 	gchar *name = NULL;
 	gint css = -1;
+	gint zoom = -1;
 
 	/* parse each attribute */
 	for (attr = attrs; *attr != NULL; attr += 2)
 	{
 		if (xmlStrcmp (attr[SAX_KEY], "name") == 0)
 		{
-			name = xml_decode_entity (attr[SAX_VALUE]);
+			name = g_strdup (attr[SAX_VALUE]);
 		}
 		else if (xmlStrcmp (attr[SAX_KEY], "authorcss") == 0)
 		{
 			css = atoi (attr[SAX_VALUE]);
 		}
+		else if (xmlStrcmp (attr[SAX_KEY], "zoom") == 0)
+		{
+			zoom = atoi (attr[SAX_VALUE]);
+		}
 		else
 		{
 			g_warning ("Unsupported attribute for "
@@ -277,6 +230,7 @@ history_parse_host_element (GlobalHistory *gh, const xmlChar **attrs)
 
 	/* create a host structure and store */
 	host = history_add_host (gh, name);
+	host->priv->zoom = zoom;
 	g_free (name);
 }
 
@@ -288,7 +242,7 @@ history_parse_item_element (GlobalHistory *gh, const xmlChar **attrs)
 {
 	const xmlChar **attr;
 	HistoryItem *item;
-	gchar *title_utf8 = NULL;
+	gchar *title = NULL;
 	gchar *url = NULL;
 	GTime first = 0;
 	GTime last = 0;
@@ -299,11 +253,11 @@ history_parse_item_element (GlobalHistory *gh, const xmlChar **attrs)
 	{
 		if (xmlStrcmp(attr[SAX_KEY], "title") == 0)
 		{
-			title_utf8 = xml_decode_entity (attr[SAX_VALUE]);
+			title = g_strdup (attr[SAX_VALUE]);
 		}
 		else if (xmlStrcmp(attr[SAX_KEY], "url") == 0)
 		{
-			url = xml_decode_entity (attr[SAX_VALUE]);
+			url = g_strdup (attr[SAX_VALUE]);
 		}
 		else if (xmlStrcmp(attr[SAX_KEY], "first_time") == 0)
 		{
@@ -329,9 +283,11 @@ history_parse_item_element (GlobalHistory *gh, const xmlChar **attrs)
 	if (calculate_age (last) <= gh->priv->expire_days)
 	{
 		/* add to the history */
-		item = history_add_item (gh, url, title_utf8, 
+		item = history_add_item (gh, url, title,
 					 first, last, visits);
 
+		g_return_if_fail (item);
+
 		for (attr = attrs; *attr != NULL; attr += 2)
 		{
 			if (xmlStrcmp(attr[SAX_KEY], "prop:") == 0)
@@ -346,7 +302,7 @@ history_parse_item_element (GlobalHistory *gh, const xmlChar **attrs)
 	else
 	{
 		g_free (url);
-		g_free (title_utf8);			
+		g_free (title);			
 	}
 
 }
@@ -418,61 +374,58 @@ history_load (GlobalHistory *gh)
 		NULL, /* error      (FIXME)    */
 		NULL, /* fatalError (FIXME)    */
 	};
-        gchar *confdir;
         gchar *histfile;
 
 	/* build the filename */
-        confdir = g_build_filename (g_get_home_dir (), GALEON_DIR, NULL);
-	histfile = g_build_filename (confdir, "history.xml", NULL);
+	histfile = g_build_filename (g_get_home_dir (), GALEON_DIR, 
+				     "history.xml", NULL);
 	
 	/* build the initial history hash table */
-        gh->priv->history_sites = g_hash_table_new (g_str_hash, g_str_equal);
+        gh->priv->history_sites = g_hash_table_new_full (g_str_hash, g_str_equal,
+							 NULL, (GDestroyNotify)history_item_free);
 
 	/* build the initial history hosts hashtable */
-	gh->priv->history_hosts = g_hash_table_new (g_str_hash, g_str_equal);
+	gh->priv->history_hosts = g_hash_table_new_full (g_str_hash, g_str_equal,
+							 NULL, (GDestroyNotify)history_item_free);
 
 	/* find out how many days to keep before expiring elements */
 	gh->priv->expire_days = eel_gconf_get_integer (CONF_HISTORY_EXPIRE);
 
-	DEBUG_MSG (("Loading history.\n"));
-	
+	LOG ("Loading history.");
+	START_PROFILER ("Loading History");
+
 	/* load the file if we can access it */
 	if (access (histfile, F_OK) != -1)
 	{
+		xmlSubstituteEntitiesDefault (1);
 		xmlSAXParseFileWithData (&parser, histfile, TRUE, gh);
 	}
+
+	STOP_PROFILER ("Loading History");
 	
 	/* free allocated strings */
-	g_free (confdir);
 	g_free (histfile);
 
-	DEBUG_MSG (("\nHistory loaded: %d sites, %d hosts.\n", 
-		    g_hash_table_size (gh->priv->history_sites),
-		    g_hash_table_size (gh->priv->history_hosts)));
+	LOG ("History loaded: %d sites, %d hosts.", 
+	     g_hash_table_size (gh->priv->history_sites),
+	     g_hash_table_size (gh->priv->history_hosts));
 }
 
-/**
- * eputs: encode and fputs an string.
- */
-static void
-eputs (guchar *text, FILE *fp)
-{
-	gchar *encoded = xmlEncodeSpecialChars (NULL, text);
-	fputs (encoded, fp);
-	xmlFree (encoded);
-}
 
 static void
 history_save_item_property (gpointer  key,
 			    gpointer  value,
 			    gpointer  user_data)
 {
-	FILE *fp = (FILE *) user_data;
+	xmlTextWriterPtr writer = (xmlTextWriterPtr)user_data;
 
-	fputs ("\" ", fp);
-	fprintf (fp, (gchar*)key);
-	fputs ("=\"prop:", fp);
-	fprintf (fp, (gchar*)value);
+	g_return_if_fail (key);
+	g_return_if_fail (value);
+
+	/* I'm not convinced that the person that wrote this code
+	 * originally meant to put the "prop:" bit before the value
+	 * - crispin 30/03/2004 */
+	xmlTextWriterWriteFormatAttribute (writer, (gchar*)key, "prop:%s", (gchar*)value);
 }
 
 /**
@@ -482,16 +435,25 @@ static void
 history_save_host (gpointer key, gpointer value, gpointer user_data)
 {
         HistoryItem *hh = (HistoryItem *) value;
-	FILE *fp = (FILE *) user_data;
+	xmlTextWriterPtr writer = (xmlTextWriterPtr)user_data;
 
-	fputs ("  <host name=\"", fp);
-	fputs (hh->url, fp);
+	g_return_if_fail (hh->url);
+
+	if (!hh->priv->children) return;
+
+	xmlTextWriterStartElement (writer, "host");
+	xmlTextWriterWriteAttribute (writer, "name", hh->url);
+
+	if (hh->priv->zoom > 0)
+	{
+		xmlTextWriterWriteFormatAttribute (writer, "zoom", "%d", hh->priv->zoom);
+	}
 
 	g_hash_table_foreach (hh->priv->properties,
 			      (GHFunc)history_save_item_property,
-			      user_data);
+			      writer);
 	
-	fputs ("\"/>\n", fp);
+	xmlTextWriterEndElement (writer);
 }
 
 /**
@@ -501,24 +463,51 @@ static void
 history_save_item (gpointer key, gpointer value, gpointer user_data)
 {
         HistoryItem *hi = (HistoryItem *) value;
-	FILE *fp = (FILE *) user_data;
+	xmlTextWriterPtr writer = (xmlTextWriterPtr)user_data;
 
-	fputs ("  <item title=\"", fp);
-	eputs (hi->title, fp);
-	fputs ("\" url=\"", fp);
-	eputs (hi->url, fp);
-	fputs ("\" first_time=\"", fp);
-	fprintf (fp, "%d", hi->first);
-	fputs ("\" last_time=\"", fp);
-	fprintf (fp, "%d", hi->last);
-	fputs ("\" visits=\"", fp);
-	fprintf (fp, "%d", hi->visits);
+	g_return_if_fail (hi->title);
+	g_return_if_fail (hi->url);
+
+	xmlTextWriterStartElement (writer, "item");
+
+	xmlTextWriterWriteAttribute (writer, "title", hi->title);
+	xmlTextWriterWriteAttribute (writer, "url", hi->url);
+
+	xmlTextWriterWriteFormatAttribute (writer, "first_time", "%d", hi->first);
+	xmlTextWriterWriteFormatAttribute (writer, "last_time", "%d", hi->last);
+	xmlTextWriterWriteFormatAttribute (writer, "visits", "%d", hi->visits);
 
 	g_hash_table_foreach (hi->priv->properties,
 			      (GHFunc)history_save_item_property,
-			      user_data);
+			      writer);
+	
+	xmlTextWriterEndElement (writer);
+}
 
-	fputs ("\"/>\n", fp);
+
+/**
+ * history_save_to_file: Actually do the file save
+ */
+static int
+history_save_to_file (GlobalHistoryPrivate *ghpriv, xmlTextWriterPtr writer)
+{
+	int ret;
+
+	ret = xmlTextWriterStartDocument (writer, "1.0", NULL, NULL);
+	if (ret < 0) return ret;
+	ret = xmlTextWriterStartElement (writer, "history");
+	if (ret < 0) return ret;
+
+        g_hash_table_foreach (ghpriv->history_hosts, history_save_host, writer);
+        g_hash_table_foreach (ghpriv->history_sites, history_save_item, writer);
+
+	ret = xmlTextWriterEndElement (writer); /* root */
+	if (ret < 0) return ret;
+
+	ret = xmlTextWriterEndDocument (writer);
+	if (ret < 0) return ret;
+	
+	return ret;
 }
 
 /** 
@@ -527,31 +516,47 @@ history_save_item (gpointer key, gpointer value, gpointer user_data)
 static void 
 history_save (GlobalHistoryPrivate *ghpriv)
 {
-        gchar *histfile;
-	FILE *fp;
+	gchar *histfile, *tmpfile;
+	xmlTextWriterPtr writer;
+	int ret;
 
-	DEBUG_MSG (("saving history\n"));
+	LOG ("saving history");
 
 	if (!ghpriv->history_dirty)
 	{
 		return;
 	}
 
-	DEBUG_MSG (("really\n"));
+	histfile = g_build_filename (g_get_home_dir (),
+				     GALEON_DIR,
+				     "history.xml",
+				     NULL);
+	tmpfile = g_strconcat (histfile, ".tmp", NULL);
 
-	histfile = g_strconcat (g_get_home_dir (), 
-				"/"GALEON_DIR"/history.xml", NULL);
-	fp = fopen (histfile, "wb");
-	g_free (histfile);
-	g_return_if_fail (fp != NULL);
+	writer = xmlNewTextWriterFilename (tmpfile, 0);
+	xmlTextWriterSetIndent (writer, 1);
+	xmlTextWriterSetIndentString (writer, "  ");
+
+	START_PROFILER ("Saving History");
+
+	ret = history_save_to_file (ghpriv, writer);
+
+	STOP_PROFILER ("Saving History");
+
+	if (ret < 0)
+	{
+		g_warning( "Failed writing to history.xml" );
+	}
+	else
+	{
+		gul_general_switch_temp_file (histfile, tmpfile);
+	}
 
-	fputs ("<?xml version=\"1.0\"?>\n", fp);
-	fputs ("<history>\n", fp);
-        g_hash_table_foreach (ghpriv->history_hosts, history_save_host, fp);
-        g_hash_table_foreach (ghpriv->history_sites, history_save_item, fp);
-	fputs ("</history>\n", fp);
-	fclose (fp);
 	ghpriv->history_dirty = FALSE;
+
+	xmlFreeTextWriter (writer);
+	g_free (histfile);
+	g_free (tmpfile);
 }
 
 void 
@@ -576,13 +581,11 @@ history_periodic_save_cb (GlobalHistoryPrivate *ghpriv)
 }
 
 /**
- * history_item_free_cb: free one history item
+ * history_item_free: free one history item
  */
 static void
-history_item_free_cb (gpointer key, gpointer value, GlobalHistory *gh)
+history_item_free (HistoryItem *hi)
 {
-	HistoryItem *hi = (HistoryItem *)value;
-
 	if (!hi->priv->is_host)
 	{
 		HistoryItem *host = hi->priv->host;
@@ -592,17 +595,13 @@ history_item_free_cb (gpointer key, gpointer value, GlobalHistory *gh)
 		}
 	}
 	
-	/* if this was the last visited item, make it NULL. 
-	 * It will be recalculated when needed */
-	if (hi == gh->priv->last_visited_site)
-	{
-		gh->priv->last_visited_site = NULL;
-	}
-
 	g_free (hi->url); 
 	g_free (hi->title);
 
 	g_hash_table_destroy (hi->priv->properties);
+
+	g_free (hi->priv);
+	g_free (hi);
 }
 
 static void
@@ -613,38 +612,34 @@ global_history_finalize (GObject *object)
         g_return_if_fail (object != NULL);
         g_return_if_fail (IS_GLOBAL_HISTORY (object));
 
-	DEBUG_MSG (("in global_history_finalize"));
+	LOG ("Finalizing GlobalHistory");
 
 	gh = GLOBAL_HISTORY (object);
 
 	/* save the history */
 	history_save (gh->priv);
-	
+
+	g_source_remove (gh->priv->timeout);
+
 	/* destroy hosts hash table */
-	g_hash_table_foreach_remove (gh->priv->history_sites, (GHRFunc)
-				     history_item_free_cb, gh);
 	g_hash_table_destroy (gh->priv->history_sites);
 	gh->priv->history_sites = NULL;
 
 	/* destroy sites hash table */
-	g_hash_table_foreach_remove (gh->priv->history_hosts, 
-				     (GHRFunc)history_item_free_cb,
-				     gh);
 	g_hash_table_destroy (gh->priv->history_hosts);
 	gh->priv->history_hosts = NULL;
 	gh->priv->last_visited_site = NULL;
 
         g_return_if_fail (gh->priv != NULL);
 
-        g_free (gh->priv);
 
-        G_OBJECT_CLASS (parent_class)->finalize (object);
+        G_OBJECT_CLASS (global_history_parent_class)->finalize (object);
 }
 
 static void
 global_history_init (GlobalHistory *gh)
 {
-        gh->priv = g_new0 (GlobalHistoryPrivate, 1);
+        gh->priv = GLOBAL_HISTORY_GET_PRIVATE (gh);
 	gh->priv->history_sites = NULL;
 	gh->priv->history_hosts = NULL;
 	gh->priv->history_dirty = FALSE;
@@ -654,16 +649,16 @@ global_history_init (GlobalHistory *gh)
 	history_load (gh);
 
 	/* setup the periodic history saving callback */
-	g_timeout_add (HISTORY_SAVE_INTERVAL, 
-		       (GSourceFunc)history_periodic_save_cb, 
-		       gh->priv);
+	gh->priv->timeout = g_timeout_add (HISTORY_SAVE_INTERVAL, 
+					   (GSourceFunc)history_periodic_save_cb, 
+					   gh->priv);
 }
 
 GlobalHistory *
 global_history_new (void)
 {
 	return GLOBAL_HISTORY (g_object_new 
-			       (GLOBAL_HISTORY_TYPE, NULL));
+			       (TYPE_GLOBAL_HISTORY, NULL));
 }
 
 static void
@@ -683,6 +678,8 @@ history_add_host (GlobalHistory *gh,
 {
 	HistoryItem *host;
 	
+	g_return_val_if_fail (name, NULL);
+
 	/* allocate */
 	host = g_new0 (HistoryItem, 1);
 
@@ -695,8 +692,8 @@ history_add_host (GlobalHistory *gh,
 	
 	host->priv = g_new0 (HistoryItemPrivate, 1);
 	host->priv->children = NULL;
-	host->priv->properties = g_hash_table_new (g_str_hash, 
-						   g_str_equal);
+	host->priv->properties = g_hash_table_new_full (g_str_hash, g_str_equal,
+							g_free, g_free);
 	
 	/* add to the hosts hashtable */
 	g_hash_table_insert (gh->priv->history_hosts, host->url, host);
@@ -717,7 +714,7 @@ history_get_host (GlobalHistory *gh,
 	HistoryItem *host;
 
 	/* check args */
-	g_assert (url != NULL);
+	g_return_val_if_fail (url, NULL);
 
  	/* check if it's a local file */
  	if (!g_ascii_strncasecmp (url, "file://", 7))
@@ -775,11 +772,16 @@ history_add_item (GlobalHistory *gh,
 	HistoryItem *hi;
 	HistoryItem *host;
 
+	g_return_val_if_fail (IS_GLOBAL_HISTORY (gh), NULL);
+	g_return_val_if_fail (url, NULL);
+	g_return_val_if_fail (title, NULL);
+
 	/* allocate */
 	hi = g_new0 (HistoryItem, 1);
 
 	/* find the parent host */
 	host = history_get_host (gh, url);
+	g_return_val_if_fail (host, NULL);
 
 	/* fill in the fields */
 	hi->url = url;
@@ -790,16 +792,23 @@ history_add_item (GlobalHistory *gh,
 	hi->priv = g_new0 (HistoryItemPrivate, 1);
 	hi->priv->children = NULL;
 	hi->priv->host = host;
-	hi->priv->properties = g_hash_table_new (g_str_hash, 
-						 g_str_equal);
+	hi->priv->properties = g_hash_table_new_full (g_str_hash, g_str_equal,
+						      g_free, g_free);
 	
 	/* update the host */
 	host->visits += visits;
 
 	/* first item of host */
-	host->first = first;
-	host->last = last;
-	host->priv->children = g_slist_append (host->priv->children, hi);
+	if (!host->first || first < host->first)
+	{
+		host->first = first;
+	}
+	
+	if (last > host->last)
+	{
+		host->last = last;
+	}
+	host->priv->children = g_slist_prepend (host->priv->children, hi);
 
 	/* add to the table */
         g_hash_table_insert (gh->priv->history_sites, 
@@ -850,7 +859,10 @@ global_history_visited (GlobalHistory *gh,
 				       g_strdup (_("Untitled")),
 				       now, now, 1);
 
-		g_signal_emit (G_OBJECT (gh), global_history_signals[ADD], 0, hi);
+		if (hi)
+		{
+			g_signal_emit (G_OBJECT (gh), global_history_signals[ADD], 0, hi);
+		}
         }
 
 	/* we have changed the history, so save at next checkpoint */
@@ -941,6 +953,58 @@ global_history_get_page_title (GlobalHistory *gh,
 	else return NULL;
 }
 
+
+gboolean 
+global_history_set_host_zoom (GlobalHistory *gh,
+			      const char *url, 
+			      gint zoom)
+{
+	HistoryItem *hi;
+	HistoryItem *hh;
+
+        hi = g_hash_table_lookup (gh->priv->history_sites, url);
+
+	if (hi == NULL)
+	{
+		return FALSE;
+	}
+
+	hh = hi->priv->host;
+
+	hh->priv->zoom = zoom;
+
+	/* we have changed the history, so save at next checkpoint */
+	gh->priv->history_dirty = TRUE;
+
+	g_signal_emit (G_OBJECT (gh), global_history_signals[UPDATE], 0, hh);
+	global_history_emit_changed_data_changed (gh);
+	
+	return TRUE;
+}
+
+/*
+ * history_get_host_zoom: gets the zoom setting for the host refernced in
+ * the url.
+ *
+ * Returns -1 if there is no zoom set for the host.
+ */
+gint
+global_history_get_host_zoom (GlobalHistory *gh,
+			      const char *url)
+{
+	HistoryItem *hi;
+	HistoryItem *hh;
+
+	hi = g_hash_table_lookup (gh->priv->history_sites, url);
+
+	if (!hi || !hi->priv->host) return -1;
+
+	hh = hi->priv->host;
+
+	return hh->priv->zoom;
+}
+
+
 gboolean 
 global_history_remove_url (GlobalHistory *gh,
 			   const char *url)
@@ -955,8 +1019,16 @@ global_history_remove_url (GlobalHistory *gh,
 
 	if (hi)
 	{	
+		/* if this was the last visited item, make it NULL.
+		 * It will be recalculated when needed */
+		if (hi == gh->priv->last_visited_site)
+		{
+			gh->priv->last_visited_site = NULL;
+		}
+
 		g_signal_emit (G_OBJECT (gh), global_history_signals[REMOVE], 0, hi);
-		history_item_free_cb ((gpointer)url, hi, gh);
+		g_hash_table_remove (gh->priv->history_sites, url);
+
 		global_history_emit_changed_data_changed (gh);
 		gh->priv->history_dirty = TRUE;
 		return FALSE;
@@ -975,8 +1047,10 @@ global_history_clear (GlobalHistory *gh)
 	/* no last_visited_site */
 	p->last_visited_site = NULL;
 
-	p->history_hosts = g_hash_table_new (g_str_hash, g_str_equal);
-	p->history_sites = g_hash_table_new (g_str_hash, g_str_equal);
+	p->history_hosts = g_hash_table_new_full (g_str_hash, g_str_equal,
+						  NULL, (GDestroyNotify)history_item_free);
+	p->history_sites = g_hash_table_new_full (g_str_hash, g_str_equal,
+						  NULL, (GDestroyNotify)history_item_free);
 
 	global_history_emit_changed_data_changed (gh);
 
@@ -985,17 +1059,10 @@ global_history_clear (GlobalHistory *gh)
 	history_save (gh->priv);
 
 	/* Destroy old history hash table */
-	g_hash_table_foreach (old_history_sites, 
-			      (GHFunc)history_item_free_cb, 
-			      gh);
 	g_hash_table_destroy (old_history_sites);
 
 	/* destroy old hosts hash table */
-	g_hash_table_foreach (old_history_hosts, 
-			      (GHFunc)history_item_free_cb, 
-			      gh);
 	g_hash_table_destroy (old_history_hosts);
-
 }
 
 /**
@@ -1022,6 +1089,24 @@ global_history_get_host_list (GlobalHistory *gh)
 	return list;
 }
 
+static void 
+history_add_item_to_slist (gpointer key, HistoryItem *item, GSList **list)
+{
+	*list = g_slist_prepend (*list, item);
+}
+
+static GSList *
+global_history_get_sites_slist (GlobalHistory *gh)
+{
+	GSList *list = NULL;
+
+	g_hash_table_foreach (gh->priv->history_sites, 
+			      (GHFunc) history_add_item_to_slist, &list);
+
+	return list;
+}
+
+
 void 
 global_history_set_page_property  (GlobalHistory *gh,
 				   HistoryItem *hi,
@@ -1151,7 +1236,7 @@ global_history_autocompletion_source_foreach_aux (gpointer key, gpointer value,
 		(little_hack[0], hi->url, hi->title, 
 		 /* very simple scoring function. Most recent entries get higher scores and each
 		    visit makes the item like 1.5 days younger aprox. */
-		    hi->last + (uint) (hi->visits << 17),
+		    hi->last + (guint) (hi->visits << 17),
 		    little_hack[2]);
 }
 
@@ -1180,3 +1265,124 @@ global_history_emit_changed_data_changed (GlobalHistory *gh)
 	g_signal_emit_by_name (gh, "data-changed");
 }
 
+/* Autobookmarks */
+
+static int 
+autobm_score_both (const HistoryItem *item)
+{
+	return item->last + (item->visits << 17);	
+}
+
+static gint 
+autobm_compare_both (const HistoryItem *a, const HistoryItem *b)
+{
+	int score_a = autobm_score_both (a);
+	int score_b = autobm_score_both (b);
+
+	return score_b > score_a ? 1 : -1;
+}
+
+static gint 
+autobm_compare_recently_visited (const HistoryItem *a, const HistoryItem *b)
+{
+	int score_a = a->last;
+	int score_b = b->last;
+
+	return score_b > score_a ? 1 : -1;
+}
+
+static gint 
+autobm_compare_frequently_visited (const HistoryItem *a, const HistoryItem *b)
+{
+	int score_a = a->visits;
+	int score_b = b->visits;
+
+	return score_b > score_a ? 1 : -1;
+}
+
+static HistoryItem *
+global_history_get_host_dominant_item (const HistoryItem *host, 
+				       GaleonAutoBookmarksSourceFilterFunc filter, gpointer filter_data, 
+				       GCompareFunc compare_func)
+{
+	const GSList *it;
+	host->priv->children = g_slist_sort (host->priv->children, compare_func);
+
+	for (it = host->priv->children; it; it = it->next)
+	{
+		HistoryItem *ret = it->data;
+		if (filter (ret->title, ret->url, filter_data))
+		{
+			return ret;
+		}
+	}
+	return NULL;
+}
+
+static void
+global_history_auto_bookmarks_source_get_autobookmarks	(GaleonAutoBookmarksSource *source, 
+							 GaleonAutoBookmarksSourceIteratorFunc iterator,
+							 gpointer iterator_data,
+							 GaleonAutoBookmarksSourceFilterFunc filter,
+							 gpointer filter_data,
+							 GaleonAutoBookmarksScoringMethod scoring,
+							 gboolean group_by_host)
+{
+	GlobalHistory *gh = GLOBAL_HISTORY (source);
+	GList *host_list;
+	const GList *it;
+	GSList *candidates = NULL;
+	const GSList *sit;
+	gboolean cont;
+	GCompareFunc compare_func = 
+		scoring == GALEON_AUTO_BOOKMARKS_SCORING_RECENTLY_VISITED ? (GCompareFunc) autobm_compare_recently_visited
+		: scoring == GALEON_AUTO_BOOKMARKS_SCORING_FRECUENTLY_VISITED ? (GCompareFunc) autobm_compare_frequently_visited
+		: (GCompareFunc) autobm_compare_both;
+
+	START_PROFILER ("history_generate_autobookmarks");
+
+	if (group_by_host)
+	{
+	     START_PROFILER ("history_generate_autobookmarks updating dominant_item");
+	     host_list = global_history_get_host_list (gh);
+	     for (it = host_list; it; it = it->next)
+	     {
+		  HistoryItem *host = it->data;
+		  HistoryItem *di = global_history_get_host_dominant_item (host, filter, filter_data, compare_func);
+		  if (di)
+		  {
+			  candidates = g_slist_prepend (candidates, di);
+		  }
+	     }
+	     g_list_free (host_list);
+	     STOP_PROFILER ("history_generate_autobookmarks updating dominant_item");
+	}
+	else
+	{
+	     START_PROFILER ("history_generate_autobookmarks getting list");
+	     candidates = global_history_get_sites_slist (gh);
+	     STOP_PROFILER ("history_generate_autobookmarks getting list");
+	}
+
+	START_PROFILER ("history_generate_autobookmarks sorting");
+	candidates = g_slist_sort (candidates, compare_func);
+	STOP_PROFILER ("history_generate_autobookmarks sorting");
+
+	cont = TRUE;
+	for (sit = candidates; sit && cont; sit = sit->next)
+	{
+		HistoryItem *hi = sit->data;
+
+		if (group_by_host /* already filtered */
+		    || filter (hi->title, hi->url, filter_data))
+		{
+			/* g_print ("AB: %s - %s\n", hi->title, hi->url); */
+			cont = iterator (hi->title, hi->url, iterator_data);
+		}
+	}
+
+	g_slist_free (candidates);
+
+	STOP_PROFILER ("history_generate_autobookmarks");
+}
+
