@@ -29,14 +29,14 @@ extern "C" {
 typedef struct GlobalHistory GlobalHistory;
 typedef struct GlobalHistoryClass GlobalHistoryClass;
 
-#define GLOBAL_HISTORY_TYPE             (global_history_get_type ())
-#define GLOBAL_HISTORY(obj)             (G_TYPE_CHECK_INSTANCE_CAST ((obj), GLOBAL_HISTORY_TYPE, \
+#define TYPE_GLOBAL_HISTORY             (global_history_get_type ())
+#define GLOBAL_HISTORY(obj)             (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_GLOBAL_HISTORY, \
 					 GlobalHistory))
 #define GLOBAL_HISTORY_CLASS(klass)     (G_TYPE_CHECK_CLASS_CAST ((klass), GLOBAL_HISTORY, \
 					 GlobalHistoryClass))
-#define IS_GLOBAL_HISTORY(obj)          (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GLOBAL_HISTORY_TYPE))
+#define IS_GLOBAL_HISTORY(obj)          (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_GLOBAL_HISTORY))
 #define IS_GLOBAL_HISTORY_CLASS(klass)  (G_TYPE_CHECK_CLASS_TYPE ((klass), GLOBAL_HISTORY))
-#define GLOBAL_HISTORY_GET_CLASS(obj)	(G_TYPE_INSTANCE_GET_CLASS ((obj), GLOBAL_HISTORY_TYPE, \
+#define GLOBAL_HISTORY_GET_CLASS(obj)	(G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_GLOBAL_HISTORY, \
 					 GlobalHistoryClass))
 
 typedef struct GlobalHistoryPrivate GlobalHistoryPrivate;
