@@ -1402,7 +1402,7 @@ new_pixbuf_from_widget (GtkWidget *widget)
   
   /* Create a pixmap */
   visual = gtk_widget_get_visual (window);
-  pixmap = gdk_pixmap_new (NULL, icon_width, icon_height, gdk_visual_get_best_depth());
+  pixmap = gdk_pixmap_new (NULL, icon_width, icon_height, visual->depth);
   gdk_drawable_set_colormap (GDK_DRAWABLE (pixmap), gtk_widget_get_colormap (window));
 
   /* Draw the window */
