@@ -20,35 +20,50 @@
 #endif
 
 #include "EventContext.h"
-#include "nsIDOMEventTarget.h"
-#include "nsIDocument.h"
-#include "nsIDOMHTMLInputElement.h"
-#include "nsIDOMHTMLObjectElement.h"
-#include "nsIInterfaceRequestor.h"
-#include "nsIDOMHTMLImageElement.h"
-#include "nsIDOMElement.h"
-#include "nsIDOMWindow.h"
-#include "nsIDOMXULDocument.h"
-#include "nsIURI.h"
-#include "nsNetUtil.h"
-#include "nsIDOMNSDocument.h"
-#include "nsReadableUtils.h"
-#include "nsGUIEvent.h"
-#include "nsIDOMNSEvent.h"
-#include "nsIDOMCharacterData.h"
-#include "nsIDOMHTMLButtonElement.h"
-#include "nsIDOMHTMLLabelElement.h"
-#include "nsIDOMHTMLLegendElement.h"
-#include "nsIDOMHTMLTextAreaElement.h"
-#include <gdk/gdkkeysyms.h>
-#include "nsIPrivateDOMEvent.h"
-#include "nsIDOMNSUIEvent.h"
+#include "GaleonWrapper.h"
+#include "GaleonUtils.h"
+#include "GulString.h"
+
+#include <nsIInterfaceRequestorUtils.h>
+#include <nsIDOMEventTarget.h>
+#include <nsIInterfaceRequestor.h>
+#include <nsIDOMElement.h>
+#include <nsIDOMWindow.h>
+#include <nsIDOMXULDocument.h>
+#include <nsIURI.h>
+#include <nsIDOMNSEvent.h>
+#include <nsIDOMNSHTMLElement.h>
+#include <nsIDOMCharacterData.h>
+#include <nsIDOMHTMLButtonElement.h>
+#include <nsIDOMHTMLLabelElement.h>
+#include <nsIDOMHTMLLegendElement.h>
+#include <nsIDOMHTMLTextAreaElement.h>
+#include <nsIDOMHTMLImageElement.h>
+#include <nsIDOMHTMLInputElement.h>
+#include <nsIDOMHTMLObjectElement.h>
+#include <nsIDOMHTMLAnchorElement.h>
+#include <nsIDOMHTMLAreaElement.h>
+#include <nsIDOMViewCSS.h>
+#include <nsIDOMDocumentView.h>
+#include <nsIDOMAbstractView.h>
+#include <nsIDOMNodeList.h>
+#include <nsIDOMCSSPrimitiveValue.h>
+#include <nsIDOMCSSStyleDeclaration.h>
+#include <nsIDOMCSSValue.h>
+#include <nsIDOMDocument.h>
+#include <nsIDOMEventTarget.h>
+#include <nsIDOMMouseEvent.h>
+#include <nsIDOMKeyEvent.h>
+#define MOZILLA_INTERNAL_API
+#include <nsIServiceManager.h>
+#undef MOZILLA_INTERNAL_API
+#include <nsITextToSubURI.h>
+#include <nsIWebBrowser.h>
 #include <nsISelection.h>
+#include <nsIDOM3Node.h>
+#include <nsMemory.h>
 
-#define KEY_CODE 256
-
-static nsresult ResolveBaseURL (nsIDocument *doc, const nsAString &relurl, nsACString &url);
-static nsresult ResolveDocumentURL (nsIDocument *doc, const nsAString &relurl, nsACString &url);
+#include <gdk/gdkkeysyms.h>
 
 EventContext::EventContext ()
 {
@@ -66,6 +81,24 @@ nsresult EventContext::Init (GaleonWrapper *wrapper)
 	return NS_OK;
 }
 
+nsresult EventContext::Unescape (const nsACString &aEscaped, nsAString &aUnescaped)
+{
+        if (!aEscaped.Length()) return NS_ERROR_FAILURE;
+
+	NS_ENSURE_TRUE (mWrapper, NS_ERROR_FAILURE);
+
+        nsCOMPtr<nsITextToSubURI> escaper
+                (do_CreateInstance ("@mozilla.org/intl/texttosuburi;1"));
+        NS_ENSURE_TRUE (escaper, NS_ERROR_FAILURE);
+
+	nsresult rv;
+	GulCString encoding;
+	rv = mWrapper->GetEncoding (encoding);
+        NS_ENSURE_SUCCESS (rv, rv);
+	
+        return escaper->UnEscapeNonAsciiURI (encoding, aEscaped, aUnescaped);
+}
+
 nsresult EventContext::GetEventContext (nsIDOMEventTarget *EventTarget,
 					GaleonEmbedEvent *info)
 {
@@ -90,9 +123,6 @@ nsresult EventContext::GetEventContext (nsIDOMEventTarget *EventTarget,
 
 	mDOMDocument = domDoc;
 
-	nsCOMPtr<nsIDocument> doc = do_QueryInterface(domDoc, &rv);
-	if (NS_FAILED(rv) || !doc) return NS_ERROR_FAILURE;
-
 	nsCOMPtr<nsIDOMXULDocument> xul_document = do_QueryInterface(domDoc);
 	if (xul_document)
 	{
@@ -109,14 +139,15 @@ nsresult EventContext::GetEventContext (nsIDOMEventTarget *EventTarget,
 	if (NS_SUCCEEDED(rv))
 	{
 		nsCOMPtr<nsISelection> selection;
-		domWindow->GetSelection(getter_AddRefs(selection));
+		rv = domWindow->GetSelection(getter_AddRefs(selection));
 		if (NS_SUCCEEDED(rv))
 		{
-			nsXPIDLString selText;
-			rv = selection->ToString(getter_Copies(selText));
-			if (NS_SUCCEEDED(rv) && !selText.IsEmpty())
+			PRUnichar * selText;
+			rv = selection->ToString (&selText);
+			if (NS_SUCCEEDED(rv) && selText)
 			{
-				SetStringProperty ("selection", selText);
+				SetStringProperty ("selection", GulString (selText));
+				nsMemory::Free (selText);
 			}
 		}
 	}
@@ -125,18 +156,21 @@ nsresult EventContext::GetEventContext (nsIDOMEventTarget *EventTarget,
 	rv = node->GetNodeType(&type);
 	if (NS_FAILED(rv)) return NS_ERROR_FAILURE;
 
+	PRBool has_background = PR_FALSE;
+
 	nsCOMPtr<nsIDOMHTMLElement> element = do_QueryInterface(node);
 	if ((nsIDOMNode::ELEMENT_NODE == type) && element)
 	{
-		nsAutoString tag;
-		rv = element->GetTagName(tag);
+		GulString uTag;
+		rv = element->GetLocalName(uTag);
 		if (NS_FAILED(rv)) return NS_ERROR_FAILURE;
 
-		if (tag.EqualsIgnoreCase("img"))
+		GulCString tag (uTag);
+		if (tag.EqualsAsciiIgnoreCase("img"))
 		{
 			info->context |= EMBED_CONTEXT_IMAGE;
 
-			nsAutoString img;
+			GulString img;
 			nsCOMPtr <nsIDOMHTMLImageElement> image = 
 						do_QueryInterface(node, &rv);
 			if (NS_FAILED(rv) || !image) return NS_ERROR_FAILURE;			
@@ -154,11 +188,12 @@ nsresult EventContext::GetEventContext (nsIDOMEventTarget *EventTarget,
 			rv = image->GetLongDesc (img);
 			if (NS_SUCCEEDED(rv) && !img.IsEmpty())
 			{
-				nsCAutoString imglongdesc;
-				rv = ResolveDocumentURL (doc, img, imglongdesc);
+				GulCString imglongdesc;
+				rv = ResolveBaseURL (img, imglongdesc);
+				if (NS_FAILED (rv)) return NS_ERROR_FAILURE;
 
 				SetStringProperty ("image_long_desc",
-						   NS_ConvertUTF8toUCS2(imglongdesc));
+						   imglongdesc.get());
 			}
 
 			int imgwidth, imgheight;
@@ -174,69 +209,37 @@ nsresult EventContext::GetEventContext (nsIDOMEventTarget *EventTarget,
 						   img);
 			}
 		}
-		else if (tag.EqualsIgnoreCase("input"))
+		else if (tag.EqualsAsciiIgnoreCase("input"))
 		{
-			nsCOMPtr<nsIDOMElement> element;
-			element = do_QueryInterface (node);
-			if (!element) return NS_ERROR_FAILURE;
-
-			NS_NAMED_LITERAL_STRING(attr, "type");
-			nsAutoString value;
-			element->GetAttribute (attr, value);
-
-			if (value.EqualsIgnoreCase("image"))
-			{
-				info->context |= EMBED_CONTEXT_IMAGE;
-				nsCOMPtr<nsIDOMHTMLInputElement> input;
-				input = do_QueryInterface (node);
-				if (!input) return NS_ERROR_FAILURE;
-
-				nsAutoString img;
-				rv = input->GetSrc (img);
-				if (NS_FAILED(rv)) return NS_ERROR_FAILURE;
-
-				nsCAutoString cImg;
-				rv = ResolveDocumentURL (doc, img, cImg);
-				if (NS_FAILED(rv)) return NS_ERROR_FAILURE;
-
-				SetStringProperty ("image", cImg.get());
-
-				if (NS_FAILED (rv)) return NS_ERROR_FAILURE;
-			}
-			else if (!value.EqualsIgnoreCase("radio") &&
-				 !value.EqualsIgnoreCase("submit") &&
-				 !value.EqualsIgnoreCase("hidden") &&
-				 !value.EqualsIgnoreCase("button") &&
-				 !value.EqualsIgnoreCase("checkbox"))
-			{
-				info->context |= EMBED_CONTEXT_INPUT;
-			}
+			CheckInput (node);			
 		}
-		else if (tag.EqualsIgnoreCase("textarea"))
+		else if (tag.EqualsAsciiIgnoreCase("textarea"))
 		{
 			info->context |= EMBED_CONTEXT_INPUT;
 		}
-		else if (tag.EqualsIgnoreCase("object"))
+		else if (tag.EqualsAsciiIgnoreCase("object"))
 		{
 			nsCOMPtr<nsIDOMHTMLObjectElement> object;
 			object = do_QueryInterface (node);
 			if (!element) return NS_ERROR_FAILURE;
 
-			nsAutoString value;
+			GulString value;
 			object->GetType(value);
 
+			GulCString cValue = value;
+
 			// MIME types are always lower case
-			if (Substring(value, 0, 6).Equals(NS_LITERAL_STRING("image/")))
+			if (cValue.StartsWith("image/"))
 			{
 				info->context |= EMBED_CONTEXT_IMAGE;
 				
-				nsAutoString img;
+				GulString img;
 				
 				rv = object->GetData (img);
 				if (NS_FAILED(rv)) return NS_ERROR_FAILURE;
 				
-				nsCAutoString cImg;
-				rv = ResolveDocumentURL (doc, img, cImg);
+				GulCString cImg;
+				rv = ResolveBaseURL (img, cImg);
 				if (NS_FAILED (rv)) return NS_ERROR_FAILURE;
 
 				SetStringProperty ("image", cImg.get());
@@ -247,6 +250,34 @@ nsresult EventContext::GetEventContext (nsIDOMEventTarget *EventTarget,
 				return NS_OK;
 			}
 		}
+		else if (tag.EqualsAsciiIgnoreCase("html"))
+		{
+			/* Clicked on part of the page without a <body>, so
+			 * look for a background image in the body tag */
+			nsCOMPtr<nsIDOMNodeList> nodeList;
+
+			rv = mDOMDocument->GetElementsByTagName (NS_LITERAL_STRING ("body"),
+								 getter_AddRefs (nodeList));
+			if (NS_SUCCEEDED (rv) && nodeList)
+			{
+				nsCOMPtr<nsIDOMNode> bodyNode;
+				nodeList->Item (0, getter_AddRefs (bodyNode));
+
+				GulString cssurl;
+				rv = GetCSSBackground (bodyNode, cssurl);
+				if (NS_SUCCEEDED (rv))
+				{
+					GulCString bgimg;
+					rv = ResolveBaseURL (cssurl, bgimg);
+					if (NS_FAILED (rv))
+						return NS_ERROR_FAILURE;
+
+					SetStringProperty ("background_image",
+							   bgimg.get());
+					has_background = TRUE;
+				}
+			}
+		}
 	}
 
 	/* Is page framed ? */
@@ -263,10 +294,11 @@ nsresult EventContext::GetEventContext (nsIDOMEventTarget *EventTarget,
 			NS_NAMED_LITERAL_STRING(nspace, "http://www.w3.org/1999/xlink");
 			NS_NAMED_LITERAL_STRING(localname_type, "type");
 
-			nsAutoString value;
+			GulString value;
 			dom_elem->GetAttributeNS (nspace, localname_type, value);
-
-			if (value.EqualsIgnoreCase("simple"))
+			
+			GulCString cValue = value;
+			if (cValue.EqualsAsciiIgnoreCase ("simple"))
 			{
 				info->context |= EMBED_CONTEXT_LINK;
 				NS_NAMED_LITERAL_STRING (localname_href, "href");
@@ -284,14 +316,16 @@ nsresult EventContext::GetEventContext (nsIDOMEventTarget *EventTarget,
 		element = do_QueryInterface(node);
 		if ((nsIDOMNode::ELEMENT_NODE == type) && element)
 		{
-			nsAutoString tag;
-			rv = element->GetTagName(tag);
+			GulString uTag;
+			rv = element->GetLocalName(uTag);
 			if (NS_FAILED(rv)) return NS_ERROR_FAILURE;
 
+			GulCString tag = uTag;
+
 			/* Link */
-			if (tag.EqualsIgnoreCase("a"))
+			if (tag.EqualsAsciiIgnoreCase("a"))
 			{
-				nsAutoString tmp;
+				GulString tmp;
 
 				rv = GatherTextUnder (node, tmp);
 				if (NS_SUCCEEDED(rv))
@@ -301,12 +335,24 @@ nsresult EventContext::GetEventContext (nsIDOMEventTarget *EventTarget,
 					do_QueryInterface(node);
 				anchor->GetHref (tmp);
 
-				if (StringBeginsWith (tmp, NS_LITERAL_STRING ("mailto:")))
+				GulCString href = tmp;
+				if (href.StartsWith ("mailto:"))
 				{
-					info->context |= EMBED_CONTEXT_EMAIL_LINK;
-					const nsAString &address = Substring(tmp, 7, tmp.Length()-7);
-					SetStringProperty ("email", address);
-				}	
+					href.Cut (0, 7);
+
+					// Chop off any extra parameters
+					char *str = g_strdup (href.get());
+					g_strdelimit (str, "?", '\0');
+
+					GulString unescapedHref;
+					rv = Unescape (GulDependentCString (str), unescapedHref);
+					if (NS_SUCCEEDED (rv) && unescapedHref.Length())
+					{
+						SetStringProperty ("email", unescapedHref);
+						info->context |= EMBED_CONTEXT_EMAIL_LINK;
+					}
+					g_free (str);
+				}
 				
 				if (anchor && !tmp.IsEmpty()) 
 				{
@@ -333,7 +379,9 @@ nsresult EventContext::GetEventContext (nsIDOMEventTarget *EventTarget,
 					if (NS_SUCCEEDED(rv))
 						SetStringProperty ("link_type", tmp);
 
-					if (tmp.EqualsIgnoreCase("text/smartbookmark"))
+					GulCString linkType = tmp;
+
+					if (linkType.EqualsAsciiIgnoreCase("text/smartbookmark"))
 					{
 						SetIntProperty ("link_is_smart", TRUE);
 						
@@ -346,7 +394,7 @@ nsresult EventContext::GetEventContext (nsIDOMEventTarget *EventTarget,
 
 							if (image)
 							{
-								nsAutoString img;
+								GulString img;
 								rv = image->GetSrc (img);
 								if (!NS_FAILED(rv))
 								{
@@ -362,19 +410,19 @@ nsresult EventContext::GetEventContext (nsIDOMEventTarget *EventTarget,
 				}
 			
 			}
-			else if (tag.EqualsIgnoreCase("option"))
+			else if (tag.EqualsAsciiIgnoreCase("option"))
 			{
 				info->context = EMBED_CONTEXT_NONE;
 				return NS_OK;
 			}
-			if (tag.EqualsIgnoreCase("area"))
+			else if (tag.EqualsAsciiIgnoreCase("area"))
 			{
 				info->context |= EMBED_CONTEXT_LINK;
 				nsCOMPtr <nsIDOMHTMLAreaElement> area =
 						do_QueryInterface(node, &rv);
 				if (NS_SUCCEEDED(rv) && area)
 				{
-					nsAutoString href;
+					GulString href;
 					rv = area->GetHref (href);
 					if (NS_FAILED(rv))
 						return NS_ERROR_FAILURE;
@@ -383,66 +431,29 @@ nsresult EventContext::GetEventContext (nsIDOMEventTarget *EventTarget,
 					CheckLinkScheme (href);
 				}
 			}
-			else if (tag.EqualsIgnoreCase("textarea") ||
-				 tag.EqualsIgnoreCase("input"))
+			else if (tag.EqualsAsciiIgnoreCase("input"))
+			{
+				CheckInput (node);
+			}
+			else if (tag.EqualsAsciiIgnoreCase("textarea"))
 			{
 				info->context |= EMBED_CONTEXT_INPUT;
 			}
 
-			nsCOMPtr<nsIDOMElement> domelement;
-			domelement = do_QueryInterface (node);
-			if (!domelement) return NS_ERROR_FAILURE;
-
-			PRBool has_background = PR_FALSE;
-
-			NS_NAMED_LITERAL_STRING(attr, "background");
-			nsAutoString value;
-			domelement->GetAttribute (attr, value);
-				
-			if (!value.IsEmpty())
-			{
-				nsCAutoString bgimg;
-				rv = ResolveDocumentURL (doc, value, bgimg);
-				if (NS_FAILED(rv)) return NS_ERROR_FAILURE;
-
-				SetStringProperty ("background_image", bgimg.get());
-			}
-			else
-			{
-				nsCOMPtr<nsIDOMHTMLBodyElement> bgelement;
-				bgelement = do_QueryInterface (node);
-				if (bgelement)
-				{
-					nsAutoString value;
-					bgelement->GetBackground (value);
-
-					if (!value.IsEmpty())
-					{
-						nsCAutoString bgimg;
-						rv = ResolveBaseURL (doc, value, bgimg);
-						if (NS_FAILED(rv))
-							return NS_ERROR_FAILURE;
-
-						SetStringProperty ("background_image",
-								   bgimg.get());
-						has_background = PR_TRUE;
-					}
-				}
-			}
-
 			if (!has_background)
 			{
-				nsAutoString cssurl;
+				GulString cssurl;
 				rv = GetCSSBackground (node, cssurl);
 				if (NS_SUCCEEDED (rv))
 				{
-					nsCAutoString bgimg;
-					rv = ResolveBaseURL (doc, cssurl, bgimg);
+					GulCString bgimg;
+					rv = ResolveBaseURL (cssurl, bgimg);
 					if (NS_FAILED (rv))
 						return NS_ERROR_FAILURE;
 
 					SetStringProperty ("background_image",
 							   bgimg.get());
+					has_background = TRUE;
 				}
 			}
 		}
@@ -455,52 +466,75 @@ nsresult EventContext::GetEventContext (nsIDOMEventTarget *EventTarget,
 	return NS_OK;
 }
 
-nsresult EventContext::GetCSSBackground (nsIDOMNode *node, nsAutoString& url)
+nsresult EventContext::GetCSSBackground (nsIDOMNode *node, nsAString& url)
 {
-	nsresult result;
+	nsresult rv;
 
-	nsCOMPtr<nsIDOMElementCSSInlineStyle> style;
-	style = do_QueryInterface (node);
-	if (!style) return NS_ERROR_FAILURE;
+	nsCOMPtr<nsIDOMDocumentView> docView = do_QueryInterface (mDOMDocument);
+	if (!docView) return NS_ERROR_FAILURE;
+
+	nsCOMPtr<nsIDOMAbstractView> abstractView;
+	rv = docView->GetDefaultView (getter_AddRefs (abstractView));
+	if (NS_FAILED (rv)) return rv;
+
+	nsCOMPtr<nsIDOMViewCSS> viewCSS = do_QueryInterface (abstractView);
+	if (!viewCSS) return NS_ERROR_FAILURE;
+	
+	nsCOMPtr<nsIDOMElement> element = do_QueryInterface (node);
+	if (!element) return NS_ERROR_FAILURE;
 
 	nsCOMPtr<nsIDOMCSSStyleDeclaration> decl;
-	result = style->GetStyle (getter_AddRefs(decl));
-	if (NS_FAILED(result)) return NS_ERROR_FAILURE;
+	rv = viewCSS->GetComputedStyle (element, NS_LITERAL_STRING(""),
+					    getter_AddRefs (decl));
+	if (NS_FAILED (rv)) return rv;
 
-	nsAutoString value;
-	NS_NAMED_LITERAL_STRING(prop_bgi, "background-image");
-	decl->GetPropertyValue (prop_bgi, value);
+	nsCOMPtr<nsIDOMCSSValue> CSSValue;
+	decl->GetPropertyCSSValue (NS_LITERAL_STRING ("background-image"),
+				   getter_AddRefs (CSSValue));
+	if (!CSSValue) return NS_ERROR_FAILURE;
 
-	if (value.IsEmpty())
-	{
-		NS_NAMED_LITERAL_STRING(prop_bg, "background");
-		decl->GetPropertyValue (prop_bg, value);
-		if (value.IsEmpty())
-		{
-			NS_NAMED_LITERAL_STRING(prop_bgr, "background-repeat");
-			decl->GetPropertyValue (prop_bgr, value);
-			if (value.IsEmpty())
-				return NS_ERROR_FAILURE;
-		}
-	}
+	nsCOMPtr<nsIDOMCSSPrimitiveValue> primitiveValue = 
+		do_QueryInterface (CSSValue);
+	if (!primitiveValue) return NS_ERROR_FAILURE;
+	
+	PRUint16 type;
+	rv = primitiveValue->GetPrimitiveType (&type);
+	if (NS_FAILED (rv)) return rv;
 
-	PRInt32 start, end;
-	nsAutoString cssurl;
-
-	NS_NAMED_LITERAL_STRING(startsub, "url(");
-	NS_NAMED_LITERAL_STRING(endsub, ")");
-
-	start = value.Find (startsub) + 4;
-	end = value.Find (endsub);
-
-	if (start == -1 || end == -1)
+	if (type != nsIDOMCSSPrimitiveValue::CSS_URI)
 		return NS_ERROR_FAILURE;
 
-	url.Assign(Substring (value, start, end - start));
+	rv = primitiveValue->GetStringValue (url);
+	if (NS_FAILED (rv)) return rv;
 
 	return NS_OK;
 }
 
+nsresult EventContext::GetTargetCoords (nsIDOMEventTarget *aTarget, PRInt32 *aX, PRInt32 *aY)
+{
+	/* Calculate the node coordinates relative to the widget origin */
+	nsCOMPtr<nsIDOMNSHTMLElement> elem = do_QueryInterface(aTarget);
+
+	PRInt32 x = 0, y = 0;
+	while (elem)
+	{
+		PRInt32 val;
+		elem->GetOffsetTop(&val);	y += val;
+		elem->GetScrollTop(&val);	y -= val;
+		elem->GetOffsetLeft(&val);	x += val;
+		elem->GetScrollLeft(&val);	x -= val;
+
+		nsCOMPtr<nsIDOMElement> parent;
+		elem->GetOffsetParent(getter_AddRefs(parent));
+		elem = do_QueryInterface(parent);
+	}
+	*aX = x;
+	*aY = y;
+
+	return NS_OK;
+}
+
+
 nsresult EventContext::GetMouseEventInfo (nsIDOMMouseEvent *aMouseEvent, GaleonEmbedEvent *info)
 {
 	nsresult result;
@@ -508,16 +542,56 @@ nsresult EventContext::GetMouseEventInfo (nsIDOMMouseEvent *aMouseEvent, GaleonE
 	info->keycode = 0;
 
 	// casting 32-bit guint* to PRUint16* below will break on big-endian
-	PRUint16 btn;
+	PRUint16 btn = 500;
 	aMouseEvent->GetButton (&btn);
-	info->mouse_button = (guint)btn;
 
-	// OTOH, casting only between (un)signedness is safe
-	aMouseEvent->GetScreenX ((PRInt32*)&info->x);
-	aMouseEvent->GetScreenY ((PRInt32*)&info->y);
+	/* Mozilla's buttons are 0 = left, 1 = middle, 2 = right,
+	 * gtk is 1 = left, 2 = middle, 3 = right */
+	switch (btn)
+	{
+		case 0:
+	        case 1:
+	        case 2:
+			info->mouse_button = btn+1;
+		break;
 
-	/* be sure we are not clicking on the scroolbars */
+#ifdef HAVE_BROKEN_CTX_MENU_EVENT
+		/* This only appears to happen when getting a mouse context menu
+		 * signal, so map it to button 3 (right mouse button)
+		 * - http://bugzilla.mozilla.org/show_bug.cgi?id=258193 */
+	        case 500: 
+			info->mouse_button = 3;
+		break;
+#endif
 
+		case (PRUint16) -1:
+			/* when the user submits a form with Return, mozilla synthesises
+			 * a _mouse_ click event with btn=65535 (-1).
+			 */
+			info->mouse_button = 0;
+		break;
+
+		default:
+			g_warning ("Unknown mouse button: %d", (int)btn);
+			info->mouse_button = 0;
+	}
+
+
+	nsCOMPtr<nsIDOMEventTarget> EventTarget;
+	result = aMouseEvent->GetTarget(getter_AddRefs(EventTarget));
+	if (NS_FAILED(result) || !EventTarget) return NS_ERROR_FAILURE;
+
+	if (info->mouse_button)
+	{
+		// OTOH, casting only between (un)signedness is safe
+		aMouseEvent->GetScreenX ((PRInt32*)&info->x);
+		aMouseEvent->GetScreenY ((PRInt32*)&info->y);
+	} else {
+		GetTargetCoords (EventTarget, (PRInt32*)&info->x,
+				 (PRInt32*)&info->y);
+	}
+
+	/* be sure we are not clicking on the scrollbars */
 	nsCOMPtr<nsIDOMNSEvent> nsEvent = do_QueryInterface(aMouseEvent, &result);
 	if (NS_FAILED(result) || !nsEvent) return NS_ERROR_FAILURE;
 
@@ -528,19 +602,18 @@ nsresult EventContext::GetMouseEventInfo (nsIDOMMouseEvent *aMouseEvent, GaleonE
 	nsCOMPtr<nsIDOMNode> OriginalNode = do_QueryInterface(OriginalTarget);
 	if (!OriginalNode) return NS_ERROR_FAILURE;
 
-	nsAutoString nodename;
+	GulString nodename;
 	OriginalNode->GetNodeName(nodename);
 
-	if (nodename.EqualsIgnoreCase("xul:scrollbarbutton") ||
-	    nodename.EqualsIgnoreCase("xul:thumb") ||
-	    nodename.EqualsIgnoreCase("xul:vbox") ||
-	    nodename.EqualsIgnoreCase("xul:spacer") ||
-	    nodename.EqualsIgnoreCase("xul:slider"))
-		return NS_ERROR_FAILURE;
+	GulCString cNodeName = nodename;
 
-	nsCOMPtr<nsIDOMEventTarget> EventTarget;
-	result = aMouseEvent->GetTarget(getter_AddRefs(EventTarget));
-	if (NS_FAILED(result) || !EventTarget) return NS_ERROR_FAILURE;
+	if (cNodeName.StartsWith ("xul:") &&
+	    (cNodeName.EqualsAsciiIgnoreCase ("xul:scrollbarbutton") ||
+	     cNodeName.EqualsAsciiIgnoreCase ("xul:thumb") ||
+	     cNodeName.EqualsAsciiIgnoreCase ("xul:vbox")  ||
+	     cNodeName.EqualsAsciiIgnoreCase ("xul:spacer") ||
+	     cNodeName.EqualsAsciiIgnoreCase ("xul:slider")))
+		return NS_ERROR_FAILURE;
 
 	result = GetEventContext (EventTarget, info);
 	if (NS_FAILED(result)) return result;
@@ -580,26 +653,9 @@ nsresult EventContext::GetKeyEventInfo (nsIDOMKeyEvent *aKeyEvent, GaleonEmbedEv
 	nsCOMPtr<nsIDOMEventTarget> target;
 	rv = aKeyEvent->GetTarget(getter_AddRefs(target));
 	if (NS_FAILED(rv) || !target) return NS_ERROR_FAILURE;
-
-	/* Calculate the node coordinates relative to the widget origin */
-	nsCOMPtr<nsIDOMNSHTMLElement> elem = do_QueryInterface(target, &rv);
-	if (NS_FAILED(rv)) return rv;
-
-	PRInt32 x = 0, y = 0;
-	while (elem)
-	{
-		PRInt32 val;
-		elem->GetOffsetTop(&val);	y += val;
-		elem->GetScrollTop(&val);	y -= val;
-		elem->GetOffsetLeft(&val);	x += val;
-		elem->GetScrollLeft(&val);	x -= val;
-
-		nsCOMPtr<nsIDOMElement> parent;
-		elem->GetOffsetParent(getter_AddRefs(parent));
-		elem = do_QueryInterface(parent, &rv);
-	}
-	info->x = x;
-	info->y = y;
+	
+	rv = GetTargetCoords (target, (PRInt32*)&info->x, (PRInt32*)&info->y);
+	NS_ENSURE_SUCCESS (rv, rv);
 
 	/* Context */
 	rv = GetEventContext (target, info);
@@ -654,25 +710,68 @@ nsresult EventContext::GetTargetDocument (nsIDOMDocument **domDoc)
 	return NS_OK;
 }
 
+nsresult EventContext::CheckInput (nsIDOMNode *aNode)
+{
+	const PRUnichar typeLiteral[] = { 't', 'y', 'p', 'e', '\0' };
+
+	nsCOMPtr<nsIDOMElement> element;
+	element = do_QueryInterface (aNode);
+	if (!element) return NS_ERROR_FAILURE;
+
+	GulString uValue;
+	element->GetAttribute (GulString(typeLiteral), uValue);
+
+	GulCString value = uValue;
+
+	if (value.EqualsAsciiIgnoreCase ("image"))
+	{
+		mEmbedEvent->context |= EMBED_CONTEXT_IMAGE;
+		nsCOMPtr<nsIDOMHTMLInputElement> input;
+		input = do_QueryInterface (aNode);
+		if (!input) return NS_ERROR_FAILURE;
+
+		nsresult rv;
+		GulString img;
+		rv = input->GetSrc (img);
+		if (NS_FAILED(rv)) return NS_ERROR_FAILURE;
+
+		GulCString cImg;
+		rv = ResolveBaseURL (img, cImg);
+		if (NS_FAILED(rv)) return NS_ERROR_FAILURE;
+		SetStringProperty ("image", cImg.get());
+	}
+	else if (!value.EqualsAsciiIgnoreCase ("radio") &&
+		 !value.EqualsAsciiIgnoreCase ("submit") &&
+		 !value.EqualsAsciiIgnoreCase ("reset") &&
+		 !value.EqualsAsciiIgnoreCase ("hidden") &&
+		 !value.EqualsAsciiIgnoreCase ("button") &&
+		 !value.EqualsAsciiIgnoreCase ("checkbox"))
+	{
+		mEmbedEvent->context |= EMBED_CONTEXT_INPUT;
+	}
+
+	return NS_OK;
+}
+
 nsresult EventContext::CheckLinkScheme (const nsAString &link)
 {
 	nsCOMPtr<nsIURI> uri;
-	NS_NewURI (getter_AddRefs (uri), link);
+	GaleonUtils::NewURI (getter_AddRefs (uri), link);
 	if (!uri) return NS_ERROR_FAILURE;
 
 	nsresult rv;
-	nsCAutoString scheme;
+	GulCString scheme;
 	rv = uri->GetScheme (scheme);
 	if (NS_FAILED (rv)) return NS_ERROR_FAILURE;
 
-	if (scheme.EqualsIgnoreCase ("http")  ||
-	    scheme.EqualsIgnoreCase ("https") ||
-	    scheme.EqualsIgnoreCase ("ftp")   ||
-	    scheme.EqualsIgnoreCase ("file")  ||
-	    scheme.EqualsIgnoreCase ("about") ||
-	    scheme.EqualsIgnoreCase ("data")  ||
-	    scheme.EqualsIgnoreCase ("resource")  ||
-	    scheme.EqualsIgnoreCase ("gopher"))
+	if (scheme.EqualsAsciiIgnoreCase ("http")  ||
+	    scheme.EqualsAsciiIgnoreCase ("https") ||
+	    scheme.EqualsAsciiIgnoreCase ("ftp")   ||
+	    scheme.EqualsAsciiIgnoreCase ("file")  ||
+	    scheme.EqualsAsciiIgnoreCase ("about") ||
+	    scheme.EqualsAsciiIgnoreCase ("data")  ||
+	    scheme.EqualsAsciiIgnoreCase ("resource")  ||
+	    scheme.EqualsAsciiIgnoreCase ("gopher"))
 	{
 		SetIntProperty ("link-can-open-in-new-tab", TRUE);
 	}
@@ -680,9 +779,9 @@ nsresult EventContext::CheckLinkScheme (const nsAString &link)
 }
 
 
-nsresult EventContext::GatherTextUnder (nsIDOMNode* aNode, nsString& aResult)
+nsresult EventContext::GatherTextUnder (nsIDOMNode* aNode, nsAString& aResult)
 {
-	nsAutoString text;
+	GulString text;
 	nsCOMPtr<nsIDOMNode> node;
 	aNode->GetFirstChild(getter_AddRefs(node));
 	PRUint32 depth = 1;
@@ -696,8 +795,8 @@ nsresult EventContext::GatherTextUnder (nsIDOMNode* aNode, nsString& aResult)
 		if (charData && nodeType == nsIDOMNode::TEXT_NODE)
 		{
 			/* Add this text to our collection. */
-			text += NS_LITERAL_STRING(" ");
-			nsAutoString data;
+			text += ' ';
+			GulString data;
 			charData->GetData(data);
 			text += data;
 		}
@@ -706,9 +805,9 @@ nsresult EventContext::GatherTextUnder (nsIDOMNode* aNode, nsString& aResult)
 			nsCOMPtr<nsIDOMHTMLImageElement> img(do_QueryInterface(node));
 			if (img)
 			{
-				nsAutoString altText;
+				GulString altText;
 				img->GetAlt(altText);
-				if (!altText.IsEmpty())
+				if (altText.Length())
 				{
 					text = altText;
 					break;
@@ -743,7 +842,6 @@ nsresult EventContext::GatherTextUnder (nsIDOMNode* aNode, nsString& aResult)
 				}
 				else
 				{
-					nsCOMPtr<nsIDOMNode> nextSibling;
 					parentNode->GetNextSibling(getter_AddRefs(nextSibling));
 					node = nextSibling;
 					depth--;
@@ -752,7 +850,8 @@ nsresult EventContext::GatherTextUnder (nsIDOMNode* aNode, nsString& aResult)
 		}
 	}
 
-	text.CompressWhitespace();
+	/* FIXME we should trim spaces here */
+
 	aResult = text;
 
 	return NS_OK;
@@ -787,46 +886,20 @@ nsresult EventContext::SetStringProperty (const char *name, const char *value)
 
 nsresult EventContext::SetStringProperty (const char *name, const nsAString &value)
 {
-	return SetStringProperty (name, NS_ConvertUCS2toUTF8(value).get());
+	return SetStringProperty (name, GulCString (value).get());
 }
 
-nsresult ResolveBaseURL (nsIDocument *doc, const nsAString &relurl, nsACString &url)
+nsresult EventContext::ResolveBaseURL (const nsAString &relurl, nsACString &url)
 {
-#if MOZILLA_SNAPSHOT > 13
-	nsIURI *base = doc->GetBaseURI ();
-	if (!base) return NS_ERROR_FAILURE;
-#elif MOZILLA_SNAPSHOT > 11
-	nsIURI *base = doc->GetBaseURL ();
-	if (!base) return NS_ERROR_FAILURE;
-#elif MOZILLA_SNAPSHOT > 9
 	nsresult rv;
+
+	nsCOMPtr<nsIDOM3Node> node(do_QueryInterface (mDOMDocument));
+	GulString spec;
+	node->GetBaseURI (spec);
+
 	nsCOMPtr<nsIURI> base;
-	rv = doc->GetBaseURL (getter_AddRefs(base));
-	if (NS_FAILED(rv)) return rv;
-#else
-	nsresult rv;
-	nsCOMPtr<nsIURI> base;
-	rv = doc->GetBaseURL (*getter_AddRefs(base));
-	if (NS_FAILED(rv)) return rv;
-#endif
+	rv = GaleonUtils::NewURI (getter_AddRefs(base), spec);
+	if (!base) return NS_ERROR_FAILURE;
 
-	return base->Resolve (NS_ConvertUCS2toUTF8(relurl), url);
-}
-
-nsresult ResolveDocumentURL (nsIDocument *doc, const nsAString &relurl, nsACString &url)
-{
-#if MOZILLA_SNAPSHOT > 13
-	nsIURI *uri = doc->GetDocumentURI ();
-	if (!uri) return NS_ERROR_FAILURE;
-#elif MOZILLA_SNAPSHOT > 11
-	nsIURI *uri = doc->GetDocumentURL ();
-	if (!uri) return NS_ERROR_FAILURE;
-#else
-	nsresult rv;
-	nsCOMPtr<nsIURI> uri;
-	rv = doc->GetDocumentURL(getter_AddRefs(uri));
-	if (NS_FAILED(rv)) return rv;
-#endif
-
-	return uri->Resolve (NS_ConvertUCS2toUTF8(relurl), url);
+	return base->Resolve (GulCString (relurl), url);
 }
