@@ -23,11 +23,8 @@
 #include "CookiePromptService.h"
 #include "ContentHandler.h"
 #include "FilePicker.h"
-#include "FtpProtocolHandler.h"
 #include "GlobalHistory.h"
-#include "IRCProtocolHandler.h"
 #include "JSConsoleService.h"
-#include "MailtoProtocolHandler.h"
 #include "MyportalProtocolHandler.h"
 #include "PrintingPromptService.h"
 #include "ProgressListener.h"
@@ -36,21 +33,22 @@
 #include "GtkNSSDialogs.h"
 #include "GtkNSSKeyPairDialogs.h"
 #include "GtkNSSClientAuthDialogs.h"
+#include "GtkNSSSecurityWarningDialogs.h"
 #endif
 
 #include "ExternalProtocolService.h"
 #include "gUrlCID.h"
 #include "nsAboutRedirector.h"
+#include "nsMailtoUrl.h"
 
 #include <nsIGenericFactory.h>
 #include <nsIComponentRegistrar.h>
 #include <nsICategoryManager.h>
 #include <nsCOMPtr.h>
-#include <nsILocalFile.h>
-#include <nsXPCOMCID.h>
-#if MOZILLA_SNAPSHOT > 13
+#define MOZILLA_INTERNAL_API
+#include <nsIServiceManager.h>
+#undef MOZILLA_INTERNAL_API
 #include <nsDocShellCID.h>
-#endif
 
 #include <glib.h>
 
@@ -61,15 +59,39 @@ NS_GENERIC_FACTORY_CONSTRUCTOR(GContentHandler)
 NS_GENERIC_FACTORY_CONSTRUCTOR(MozGlobalHistory)
 NS_GENERIC_FACTORY_CONSTRUCTOR(GPrintingPromptService)
 NS_GENERIC_FACTORY_CONSTRUCTOR(GSidebarProxy)
+NS_DECL_CLASSINFO(GSidebarProxy)
 NS_GENERIC_FACTORY_CONSTRUCTOR(GCookiePromptService)
+NS_GENERIC_FACTORY_CONSTRUCTOR(nsAboutRedirector)
 #ifdef HAVE_MOZILLA_PSM
 NS_GENERIC_FACTORY_CONSTRUCTOR(GtkNSSDialogs)
 NS_GENERIC_FACTORY_CONSTRUCTOR(GtkNSSKeyPairDialogs)
 NS_GENERIC_FACTORY_CONSTRUCTOR(GtkNSSClientAuthDialogs)
+NS_GENERIC_FACTORY_CONSTRUCTOR(GtkNSSSecurityWarningDialogs)
 #endif
 NS_GENERIC_FACTORY_CONSTRUCTOR(GMyportalProtocolHandler)
-NS_GENERIC_FACTORY_CONSTRUCTOR(GIRCProtocolHandler)
-NS_GENERIC_FACTORY_CONSTRUCTOR(GFtpProtocolHandler)
+NS_GENERIC_FACTORY_CONSTRUCTOR(GExternalProtocolService)
+
+NS_GENERIC_FACTORY_CONSTRUCTOR(nsMailtoUrl);
+static NS_DEFINE_CID(kMailtoUrlCID, NS_MAILTOURL_CID);
+
+// NS_DOWNLOAD_CONTRACTID was renamed in mozilla 1.8b
+#ifndef NS_TRANSFER_CONTRACTID
+#define NS_TRANSFER_CONTRACTID NS_DOWNLOAD_CONTRACTID
+#endif
+
+static NS_METHOD
+RegisterSidebar(nsIComponentManager *aCompMgr, nsIFile *aPath,
+                const char *registryLocation, const char *componentType,
+                const nsModuleComponentInfo *info)
+{
+	nsCOMPtr<nsICategoryManager> cm =
+		do_GetService(NS_CATEGORYMANAGER_CONTRACTID);
+	NS_ENSURE_TRUE (cm, NS_ERROR_FAILURE);
+
+	return cm->AddCategoryEntry("JavaScript global property",
+				    "sidebar", NS_SIDEBAR_CONTRACTID,
+				    PR_FALSE, PR_TRUE, nsnull);
+}
 
 static const nsModuleComponentInfo sAppComps[] = {
 	{
@@ -81,7 +103,7 @@ static const nsModuleComponentInfo sAppComps[] = {
 	{
 		G_PROGRESSDIALOG_CLASSNAME,
 		G_PROGRESSDIALOG_CID,
-		NS_DOWNLOAD_CONTRACTID,
+		NS_TRANSFER_CONTRACTID,
 		GProgressListenerConstructor
 	},
 	{	
@@ -102,21 +124,18 @@ static const nsModuleComponentInfo sAppComps[] = {
 		NS_IHELPERAPPLAUNCHERDLG_CONTRACTID,
 		GContentHandlerConstructor
 	},
-#if MOZILLA_SNAPSHOT <= 13
 	{
-		GALEON_GLOBALHISTORY_CLASSNAME,
-		GALEON_GLOBALHISTORY_CID,
-		NS_GLOBALHISTORY_CONTRACTID,
-		MozGlobalHistoryConstructor
+		G_EXTERNALPROTOCOLSERVICE_CLASSNAME,
+		G_EXTERNALPROTOCOLSERVICE_CID,
+		NS_EXTERNALPROTOCOLSERVICE_CONTRACTID,
+		GExternalProtocolServiceConstructor
 	},
-#else
 	{
 		GALEON_GLOBALHISTORY_CLASSNAME,
 		GALEON_GLOBALHISTORY_CID,
 		NS_GLOBALHISTORY2_CONTRACTID,
 		MozGlobalHistoryConstructor
 	},
-#endif
 	{
 		G_PRINTINGPROMPTSERVICE_CLASSNAME,
 		G_PRINTINGPROMPTSERVICE_CID,
@@ -127,7 +146,14 @@ static const nsModuleComponentInfo sAppComps[] = {
 		G_SIDEBAR_CLASSNAME,
 		G_SIDEBAR_CID,
 		NS_SIDEBAR_CONTRACTID,
-		GSidebarProxyConstructor
+		GSidebarProxyConstructor,
+		RegisterSidebar,
+		nsnull, // No unregister func
+		nsnull, // No factory destructor
+		NS_CI_INTERFACE_GETTER_NAME(GSidebarProxy),
+		nsnull, // No language helper
+		&NS_CLASSINFO_NAME(GSidebarProxy),
+		nsIClassInfo::DOM_OBJECT
 	},
 	{
 		G_COOKIEPROMPTSERVICE_CLASSNAME,
@@ -160,6 +186,12 @@ static const nsModuleComponentInfo sAppComps[] = {
 		NS_GENERATINGKEYPAIRINFODIALOGS_CONTRACTID,
 		GtkNSSKeyPairDialogsConstructor
 	},
+        {
+                GTK_NSSSECURITYWARNINGDIALOGS_CLASSNAME,
+                GTK_NSSSECURITYWARNINGDIALOGS_CID,
+                NS_SECURITYWARNINGDIALOGS_CONTRACTID,
+                GtkNSSSecurityWarningDialogsConstructor
+        },
 #endif
 	{
 		G_ABOUT_MYPORTAL_CLASSNAME,
@@ -174,82 +206,70 @@ static const nsModuleComponentInfo sAppComps[] = {
 		GMyportalProtocolHandlerConstructor
 	},
 	{
-		G_IRC_PROTOCOL_CLASSNAME,
-		G_IRC_PROTOCOL_CID,
-		G_IRC_PROTOCOL_CONTRACTID,
-		GIRCProtocolHandlerConstructor
+		G_ABOUT_TOPHER_MODULE_CLASSNAME,
+		G_ABOUT_REDIRECTOR_MODULE_CID,
+		G_ABOUT_TOPHER_MODULE_CONTRACTID,
+		nsAboutRedirectorConstructor
 	},
-	{
-		G_IRC_CONTENT_CLASSNAME,
-		G_IRC_PROTOCOL_CID,
-		G_IRC_CONTENT_CONTRACTID,
-		GIRCProtocolHandlerConstructor
-	},
-	{
-		G_FTP_CONTENT_CLASSNAME,
-		G_FTP_PROTOCOL_CID,
-		G_FTP_CONTENT_CONTRACTID,
-		GFtpProtocolHandlerConstructor
+	{       /* This MUST be last */
+		NS_MAILTOURL_CLASSNAME,
+		NS_MAILTOURL_CID,
+		NS_MAILTOURL_CONTRACTID,
+		nsMailtoUrlConstructor
 	}
 };
 
-static const int sNumAppComps = sizeof(sAppComps) / sizeof(nsModuleComponentInfo);
+#define NS_MSGCONTENTPOLICY_CONTRACTID "@mozilla.org/messenger/content-policy;1"
 
-static const nsModuleComponentInfo sFtpComps = {
-	G_FTP_PROTOCOL_CLASSNAME,
-	G_FTP_PROTOCOL_CID,
-	G_FTP_PROTOCOL_CONTRACTID,
-	GFtpProtocolHandlerConstructor
-};
+static nsresult
+remove_thunderbird_contentpolicy()
+{
+	nsCOMPtr<nsICategoryManager> cm;
+	cm = do_GetService(NS_CATEGORYMANAGER_CONTRACTID);
+	NS_ENSURE_TRUE (cm, NS_ERROR_FAILURE);
 
-NS_GENERIC_FACTORY_CONSTRUCTOR(GMailtoProtocolHandler)
+	cm->DeleteCategoryEntry("content-policy",
+				NS_MSGCONTENTPOLICY_CONTRACTID, PR_TRUE);
 
-static const nsModuleComponentInfo sMailtoComps[] = {
-	{
-		G_MAILTO_PROTOCOL_CLASSNAME,
-		G_MAILTO_PROTOCOL_CID,
-		G_MAILTO_PROTOCOL_CONTRACTID,
-		GMailtoProtocolHandlerConstructor
-	},
-	{
-		G_MAILTO_CONTENT_CLASSNAME,
-		G_MAILTO_PROTOCOL_CID,
-		G_MAILTO_CONTENT_CONTRACTID,
-		GMailtoProtocolHandlerConstructor
-	}
-};
+	return NS_OK;
+}
 
-static const int sNumMailtoComps = sizeof(sMailtoComps) / sizeof(nsModuleComponentInfo);
+#if defined(HAVE_MOZILLA_PSM) && ! defined(HAVE_NSIDOCSHELL_GETSECUREUI)
+/* feb0fe98-f5ee-4d26-ad76-70856de63720 */
+#define NEW_CID { 0xfeb0fe98, 0xf5ee, 0x4d26, { 0xad, 0x76, 0x70, 0x85, 0x6d, 0xe6, 0x37, 0x20 } }
 
-static const char *sModulePath = LIB_DIR"/components/libgaleon-xpcom.so";
+static nsresult
+reregister_secure_browser_ui (nsIComponentManager *cm, nsIComponentRegistrar *cr)
+{
+	nsresult rv;
+	
+	/* Workaround as a result of:
+	 *  https://bugzilla.mozilla.org/show_bug.cgi?id=94974
+	 * see
+	 *  http://bugzilla.gnome.org/show_bug.cgi?id=164670
+	 */
+	nsCOMPtr<nsIFactory> factory;
+	rv = cm->GetClassObjectByContractID ("@mozilla.org/secure_browser_ui;1", 
+					     NS_GET_IID(nsIFactory), getter_AddRefs (factory));
+	NS_ENSURE_SUCCESS (rv, rv);
 
-static const nsModuleComponentInfo sModuleComps[] = {
-	{
-		 G_EXTERNALPROTOCOLSERVICE_CLASSNAME,
-		 G_EXTERNALPROTOCOLSERVICE_CID,
-		 NS_EXTERNALPROTOCOLSERVICE_CONTRACTID
-	},
-	{
-		 G_ABOUT_TOPHER_MODULE_CLASSNAME,
-		 G_ABOUT_REDIRECTOR_MODULE_CID,
-		 G_ABOUT_TOPHER_MODULE_CONTRACTID
-	}
-};
+	nsCID *cidPtr = nsnull;
+	rv = cr->ContractIDToCID("@mozilla.org/secure_browser_ui;1", &cidPtr);
+	NS_ENSURE_SUCCESS (rv, rv);
 
-static const int sNumModuleComps = sizeof(sModuleComps) / sizeof(nsModuleComponentInfo);
+	rv = cr->UnregisterFactory (*cidPtr, factory);
+	NS_ENSURE_SUCCESS (rv, rv);
 
+	const nsCID new_cid = NEW_CID;
+	rv = cr->RegisterFactory (new_cid, "Galeon Secure Browser Class",
+				  "@galeon.sf.net/hacks/secure-browser-ui;1", factory);
+	NS_ENSURE_SUCCESS (rv, rv);
 
-static NS_DEFINE_CID(kMailtoUrlCID, NS_MAILTOURL_CID);
+	return NS_OK;
+}
+#endif
 
-static NS_DEFINE_CID(knsFtpProtocolHandlerCID, NS_FTPPROTOCOLHANDLER_CID);
-
-//Annoying globals to track the mozilla ftp handler so it can be restored.
-static PRBool ftpRegistered = PR_FALSE;
-static nsCOMPtr<nsIFactory> nsFtpFactory;
-
-/* FIXME why we need to use "C" here ???? */
-
-extern  "C" gboolean
+gboolean
 mozilla_register_components (void)
 {
 	gboolean ret = TRUE;
@@ -259,13 +279,30 @@ mozilla_register_components (void)
 	rv = NS_GetComponentRegistrar(getter_AddRefs(cr));
 	NS_ENSURE_SUCCESS(rv, rv);
 
-	for (int i = 0; i < sNumAppComps; i++)
+	nsCOMPtr<nsIComponentManager> cm;
+	NS_GetComponentManager (getter_AddRefs (cm));
+	NS_ENSURE_TRUE (cm, FALSE);
+
+	guint numAppComps = G_N_ELEMENTS (sAppComps);
+
+	/* See if we should register the nsMailtoUrl handler */
+	PRBool isMailtoUrlReg;
+	rv = cr->IsContractIDRegistered(NS_MAILTOURL_CONTRACTID,
+					&isMailtoUrlReg);
+	if (NS_SUCCEEDED(rv) && isMailtoUrlReg)
+	{
+		numAppComps--; /* No */
+	}
+
+	for (guint i = 0; i < G_N_ELEMENTS (sAppComps); i++)
 	{
 		nsCOMPtr<nsIGenericFactory> componentFactory;
 		rv = NS_NewGenericFactory(getter_AddRefs(componentFactory),
 					  &(sAppComps[i]));
-		if (NS_FAILED(rv))
+		if (NS_FAILED(rv) || !componentFactory)
 		{
+			g_warning ("Failed to make a factory for %s\n", sAppComps[i].mDescription);
+
 			ret = FALSE;
 			continue;  // don't abort registering other components
 		}
@@ -275,134 +312,31 @@ mozilla_register_components (void)
 					 sAppComps[i].mContractID,
 					 componentFactory);
 		if (NS_FAILED(rv))
+		{
+			g_warning ("Failed to register %s\n", sAppComps[i].mDescription);
+
 			ret = FALSE;
+		}
+
+		if (sAppComps[i].mRegisterSelfProc)
+		{
+			rv = sAppComps[i].mRegisterSelfProc (cm, nsnull, nsnull, nsnull, &sAppComps[i]);
+
+			if (NS_FAILED (rv))
+			{
+				g_warning ("Failed to register-self for %s\n", sAppComps[i].mDescription);
+				ret = FALSE;
+			}
+		}
 	}
 
-	nsCOMPtr<nsICategoryManager> cm;
-	cm = do_GetService(NS_CATEGORYMANAGER_CONTRACTID, &rv);
-	NS_ENSURE_SUCCESS(rv, rv);
+	// Thunderbird includes a fascist content policy - we don't need it
+	remove_thunderbird_contentpolicy ();
 
-	cm->AddCategoryEntry("JavaScript global property",
-			     "sidebar", NS_SIDEBAR_CONTRACTID,
-			     false, true, nsnull);
-
-	nsCOMPtr<nsILocalFile> dllFile;
-	rv = NS_NewLocalFile(NS_ConvertUTF8toUCS2(sModulePath), PR_TRUE,
-			     getter_AddRefs (dllFile));
-	if (NS_FAILED(rv) || !dllFile) return FALSE;
-
-	for (int i = 0 ; i < sNumModuleComps ; i++)
-	{
-		rv = cr->RegisterFactoryLocation(sModuleComps[i].mCID,
-						 sModuleComps[i].mDescription,
-						 sModuleComps[i].mContractID,
-						 dllFile, nsnull, nsnull);
-		if (NS_FAILED(rv)) ret = FALSE;
-	}
-
-	PRBool isMailtoUrlReg;
-	rv = cr->IsContractIDRegistered(NS_MAILTOURL_CONTRACTID,
-					&isMailtoUrlReg);
-	if (NS_SUCCEEDED(rv) && isMailtoUrlReg) return ret;
-
-	rv = cr->RegisterFactoryLocation(kMailtoUrlCID,
-					 NS_MAILTOURL_CLASSNAME,
-					 NS_MAILTOURL_CONTRACTID,
-					 dllFile, nsnull, nsnull);
-	if (NS_FAILED(rv)) ret = FALSE;
+#if defined(HAVE_MOZILLA_PSM) && ! defined(HAVE_NSIDOCSHELL_GETSECUREUI)
+	// Workaround for http://bugzilla.gnome.org/show_bug.cgi?id=164670
+	reregister_secure_browser_ui (cm, cr);
+#endif
 
 	return ret;
 }
-
-/**
- * mozilla_register_FtpProtocolHandler: Register Ftp Protocol Handler
- */
-extern "C" gboolean mozilla_register_FtpProtocolHandler (void)
-{
-	if (ftpRegistered == PR_TRUE) return TRUE;
-
-	nsresult rv = NS_OK;
-
-	nsCOMPtr<nsIComponentManager> cm;
-	rv = NS_GetComponentManager(getter_AddRefs(cm));
-	if (NS_FAILED(rv) || !cm) return FALSE;
-
-	rv = cm->GetClassObject(knsFtpProtocolHandlerCID,
-				NS_GET_IID(nsIFactory),
-				getter_AddRefs(nsFtpFactory));
-        if (NS_FAILED(rv)) return FALSE;
-
-	nsCOMPtr<nsIGenericFactory> ftpFactory;
-	rv = NS_NewGenericFactory(getter_AddRefs(ftpFactory),
-				  &sFtpComps);
-	if (NS_FAILED(rv) || !ftpFactory) return FALSE;
-
-	nsCOMPtr<nsIComponentRegistrar> cr;
-	rv = NS_GetComponentRegistrar(getter_AddRefs(cr));
-	if (NS_FAILED(rv) || !cr) return FALSE;
-
-	rv = cr->RegisterFactory(sFtpComps.mCID,
-				 sFtpComps.mDescription,
-				 sFtpComps.mContractID,
-				 ftpFactory);  
-	if (NS_FAILED(rv)) return FALSE;
-
-	ftpRegistered = PR_TRUE;
-	return NS_SUCCEEDED (rv) ? TRUE : FALSE;
-}
-
-/**
- * mozilla_unregister_FtpProtocolHandler: Unregister Ftp Protocol Handler
- */
-extern "C" gboolean mozilla_unregister_FtpProtocolHandler (void)
-{
-	if (ftpRegistered == PR_FALSE) return FALSE;
-        
-        nsresult rv = NS_OK;
-
-	nsCOMPtr<nsIComponentRegistrar> cr;
-	rv = NS_GetComponentRegistrar(getter_AddRefs(cr));
-	if (NS_FAILED(rv) || !cr) return FALSE;
-
-	rv = cr->RegisterFactory(knsFtpProtocolHandlerCID,
-				 NS_FTPPROTOCOLHANDLER_CLASSNAME,
-				 G_FTP_PROTOCOL_CONTRACTID,
-				 nsFtpFactory);
-
-	ftpRegistered = PR_FALSE;
-        return NS_SUCCEEDED (rv) ? TRUE : FALSE;
-}
-
-/**
- * mozilla_register_MailtoProtocolHandler: Register Mailto Protocol Handler
- */
-extern "C" gboolean 
-mozilla_register_MailtoProtocolHandler (void)
-{
-	gboolean retVal = TRUE;
-        nsresult rv;
-
-	nsCOMPtr<nsIComponentRegistrar> cr;
-	rv = NS_GetComponentRegistrar(getter_AddRefs(cr));
-	if (NS_FAILED(rv) || !cr) return FALSE;
-
-	for (int i = 0; i < sNumMailtoComps; i++)
-	{
-		nsCOMPtr<nsIGenericFactory> componentFactory;
-		rv = NS_NewGenericFactory(getter_AddRefs(componentFactory),
-					  &(sMailtoComps[i]));
-		if (NS_FAILED(rv))
-		{
-			retVal = FALSE;
-			continue;  // don't abort registering other components
-		}
-
-		rv = cr->RegisterFactory(sMailtoComps[i].mCID,
-					 sMailtoComps[i].mDescription,
-					 sMailtoComps[i].mContractID,
-					 componentFactory);
-		if (NS_FAILED(rv))
-			retVal = FALSE;
-	}
-	return retVal;
-}
