@@ -33,55 +33,50 @@
 #include "eel-gconf-extensions.h"
 #include "galeon-embed-prefs.h"
 #include "MozRegisterComponents.h"
-#include "SideBar.h"
-#include "nsIConsoleService.h"
 #include "JSConsoleListener.h"
-#include "nsIJSConsoleService.h"
 #include "CookieObserver.h"
 #include "mozilla-cookie.h"
 #include "mozilla-encodings.h"
+#include "GaleonUtils.h"
+#include "MozillaPrivate.h"
+#include "GulString.h"
 
 #include <time.h>
 #include <glib/gi18n.h>
 #include <string.h>
+
+#include <nsIConsoleService.h>
+#include <nsIJSConsoleService.h>
 #include <nsICacheService.h>
 #include <nsCOMPtr.h>
 #include <nsNetCID.h>
-#include <nsNetUtil.h>
+#define MOZILLA_INTERNAL_API
 #include <nsIServiceManager.h>
+#undef MOZILLA_INTERNAL_API
 #include <nsIIOService.h>
-#include <nsIProtocolProxyService.h>
 #include <nsIJVMManager.h>
-#include <nsIAtom.h>
-#include <nsICharsetConverterManager.h>
-#if MOZILLA_SNAPSHOT > 9
-#include <nsIStringEnumerator.h>
-#else
-#include <nsICharsetConverterManager2.h>
-#endif
 #include <nsIFontEnumerator.h>
 #include <nsISupportsPrimitives.h>
-#include <nsReadableUtils.h>
 #include <nsIPermissionManager.h>
 #include <nsICookieManager.h>
 #include <nsIPermission.h>
 #include <nsIPasswordManager.h>
 #include <nsIPassword.h>
 #include <nsICookie.h>
-#if MOZILLA_SNAPSHOT <= 12
-#include <nsCCookieManager.h>
-#endif
 #include <nsCPasswordManager.h>
 #include <nsIObserverService.h>
 #include <nsIPrefService.h>
+#include <nsILocalFile.h>
+#include <nsIURI.h>
+#include <nsMemory.h>
 
 //For setting the locale. hopefully gtkmozembed will do itself soon
-#if MOZILLA_SNAPSHOT > 13
 #include <nsIChromeRegistry.h>
-#else
-#include <chrome/nsIChromeRegistry.h>
-#endif
 #include <locale/nsILocaleService.h>
+#ifdef HAVE_NSICHROMEREGISTRYSEA_H
+# include <chrome/nsIChromeRegistrySea.h>
+#endif
+
 
 #define MOZILLA_PROFILE_DIR  GALEON_DIR"/mozilla"
 #define MOZILLA_PROFILE_NAME "galeon"
@@ -92,158 +87,26 @@
 #define DEFAULT_PROFILE_FILE SHARE_DIR"/default-prefs.js"
 #endif
 
-#if MOZILLA_SNAPSHOT > 9
 //Table of permission types to map to new string based api
 const char* gPermType[3] = { "cookie", "image", "popup" };
-#endif
-
-static void
-mozilla_embed_shell_class_init (MozillaEmbedShellClass *klass);
-static void
-mozilla_embed_shell_init (MozillaEmbedShell *ges);
-static void
-mozilla_embed_shell_finalize (GObject *object);
-
-static gboolean
-impl_initialize (GaleonEmbedShell *mes);
-
-static void              
-impl_get_capabilities (GaleonEmbedShell *shell,
-		       EmbedShellCapabilities *caps);
-static gresult      
-impl_clear_cache (GaleonEmbedShell *shell,
-		  CacheType type);
-static gresult          
-impl_set_offline_mode (GaleonEmbedShell *shell,
-		       gboolean offline);
-static gresult           
-impl_load_proxy_autoconf (GaleonEmbedShell *shell,
-			  const char* url);
-static gresult           
-impl_show_java_console (GaleonEmbedShell *shell);
-static gresult           
-impl_shell_show_js_console (GaleonEmbedShell *shell);
-static gresult           
-impl_get_encodings (GaleonEmbedShell *shell,
-		    GaleonEncodings **encodings);
-static gresult           
-impl_get_font_langs (GaleonEmbedShell *shell,
-		     GList **langs);
-static gresult
-impl_get_font_list (GaleonEmbedShell *shell,
-		    const char *langGroup,
-		    const char *fontType,
-		    GList **fontList,
-		    char **default_font);
-static gresult           
-impl_set_permission (GaleonEmbedShell *shell,
-		     const char *url, 
-		     PermissionType type,
-		     gboolean allow);
-static gresult           
-impl_list_permissions (GaleonEmbedShell *shell,
-		       PermissionType type, 
-		       GList **permissions);
-static gresult           
-impl_remove_permissions (GaleonEmbedShell *shell,
-		         PermissionType type,
-		         GList *permissions);
-static gresult           
-impl_test_permission (GaleonEmbedShell *shell,
-		      const char *url, 
-		      PermissionType type,
-		      PermissionActionType *ret_action);
-static gresult
-impl_list_cookies (GaleonEmbedShell *shell,
-		   GList **cookies);
-static gresult           
-impl_remove_cookies (GaleonEmbedShell *shell,
-		     GList *cookies,
-		     gboolean block);
-static gresult           
-impl_list_passwords (GaleonEmbedShell *shell,
-		     PasswordType type, 
-		     GList **passwords);
-static gresult           
-impl_remove_passwords (GaleonEmbedShell *shell,
-		       GList *passwords,
-		       PasswordType type);
 
 static void mozilla_embed_shell_new_window_orphan_cb (GtkMozEmbedSingle *embed,
             	           		              GtkMozEmbed **retval, 
 					              guint chrome_mask,
                            		              GaleonEmbedShell *shell);
 
+#define MOZILLA_EMBED_SHELL_GET_PRIVATE(object) (G_TYPE_INSTANCE_GET_PRIVATE ((object), \
+				       MOZILLA_TYPE_EMBED_SHELL, MozillaEmbedShellPrivate))
+
+
 struct MozillaEmbedShellPrivate
 {
-	GaleonDialog *js_console;
 	MozillaEncodings *encodings;
 };
 
 static NS_DEFINE_CID(kJVMManagerCID, NS_JVMMANAGER_CID);
 
-GObject *mozilla_js_console;
-
-static GObjectClass *parent_class = NULL;
-
-GType
-mozilla_embed_shell_get_type (void)
-{
-       static GType mozilla_embed_shell_type = 0;
-
-        if (mozilla_embed_shell_type == 0)
-        {
-                static const GTypeInfo our_info =
-                {
-                        sizeof (MozillaEmbedShellClass),
-                        NULL, /* base_init */
-                        NULL, /* base_finalize */
-                        (GClassInitFunc) mozilla_embed_shell_class_init,
-                        NULL, /* class_finalize */
-                        NULL, /* class_data */
-                        sizeof (MozillaEmbedShell),
-                        0,    /* n_preallocs */
-                        (GInstanceInitFunc) mozilla_embed_shell_init
-                };
-
-                mozilla_embed_shell_type = g_type_register_static (GALEON_EMBED_SHELL_TYPE,
-								   "MozillaEmbedShell",
-								   &our_info, (GTypeFlags)0);
-        }
-
-        return mozilla_embed_shell_type;
-}
-
-static void
-mozilla_embed_shell_class_init (MozillaEmbedShellClass *klass)
-{
-	GObjectClass *object_class = G_OBJECT_CLASS (klass);
-	GaleonEmbedShellClass *shell_class;
-	
-	parent_class = (GObjectClass *) g_type_class_peek_parent (klass);
-	shell_class = GALEON_EMBED_SHELL_CLASS (klass);
-	
-        object_class->finalize = mozilla_embed_shell_finalize;
-
-	shell_class->initialize = impl_initialize;
-	shell_class->get_capabilities = impl_get_capabilities;
-	shell_class->clear_cache = impl_clear_cache;
-	shell_class->set_offline_mode = impl_set_offline_mode;
-	shell_class->load_proxy_autoconf = impl_load_proxy_autoconf;
-	shell_class->show_java_console = impl_show_java_console;
-	shell_class->show_js_console = impl_shell_show_js_console;
-	shell_class->get_encodings   = impl_get_encodings;
-	shell_class->get_font_langs = impl_get_font_langs;
-	shell_class->get_font_list = impl_get_font_list;
-	shell_class->set_permission = impl_set_permission;
-	shell_class->list_permissions = impl_list_permissions;
-	shell_class->remove_permissions = impl_remove_permissions;
-	shell_class->test_permission = impl_test_permission;
-	shell_class->list_cookies = impl_list_cookies;
-	shell_class->remove_cookies = impl_remove_cookies;
-	shell_class->list_passwords = impl_list_passwords;
-	shell_class->remove_passwords = impl_remove_passwords;
-}
+G_DEFINE_TYPE (MozillaEmbedShell, mozilla_embed_shell, GALEON_TYPE_EMBED_SHELL);
 
 static gboolean
 mozilla_set_default_prefs (void)
@@ -314,13 +177,19 @@ mozilla_init_single (MozillaEmbedShell *mes)
 static void
 mozilla_init_home (void)
 {
-	char *mozilla_five_home;
-        mozilla_five_home = g_strdup (g_getenv ("MOZILLA_FIVE_HOME"));
-        gtk_moz_embed_set_comp_path (mozilla_five_home);
-        g_free (mozilla_five_home);
+	char *mozilla_home;
+
+	mozilla_home = g_strdup (g_getenv ("GALEON_MOZILLA_HOME"));
+	if (mozilla_home == NULL)
+	{
+		mozilla_home = g_strdup (GALEON_MOZILLA_HOME);
+	}
+
+        gtk_moz_embed_set_comp_path (mozilla_home);
+	g_free (mozilla_home);
 }
 
-void
+static void
 mozilla_init_profile (void)
 {
 	char *profile_path;
@@ -332,7 +201,35 @@ mozilla_init_profile (void)
 }
 
 static void
-mozilla_init_js_console (MozillaEmbedShell *mes)
+mozilla_init_plugin_path ()
+{
+	const char * user_path = g_getenv( "MOZ_PLUGIN_PATH" );
+
+	GString *new_path = g_string_new (user_path);
+	
+	/* Possibly append a ':' */
+	if (new_path->len && new_path->str[new_path->len-1] != ':')
+	{
+		g_string_append_c (new_path, ':');
+	}
+
+	/* Add the user's plugin path (~/.mozilla/plugins) */
+	g_string_append (new_path, g_get_home_dir());
+	g_string_append (new_path, "/.mozilla/plugins");
+
+	/* Add /usr/lib/mozilla/plugins (bug 149812) */
+	g_string_append (new_path, ":/usr/lib/mozilla/plugins");
+
+	/* Add the current mozilla build's plugins dir */
+	g_string_append (new_path, ":" GALEON_MOZILLA_HOME "/plugins");
+
+	g_setenv ("MOZ_PLUGIN_PATH", new_path->str, TRUE);
+
+	g_string_free (new_path, TRUE);
+}
+
+static void
+mozilla_init_js_console ()
 {
 	nsCOMPtr<nsIConsoleService> consoleService = 
 		do_GetService (NS_CONSOLESERVICE_CONTRACTID);
@@ -340,10 +237,6 @@ mozilla_init_js_console (MozillaEmbedShell *mes)
 
         JSConsoleListener *listener = new JSConsoleListener();
         consoleService->RegisterListener (listener);
-	
-	mes->priv->js_console = js_console_new ();
-
-	mozilla_js_console = G_OBJECT(mes->priv->js_console);
 }
 
 #if 0
@@ -372,24 +265,6 @@ have_gnome_url_handler (const gchar *protocol)
 }
 #endif
 
-static void
-mozilla_register_external_protocols (void)
-{
-	/* FIXME register only when needed */
-#if 0
-	if (have_gnome_url_handler ("ftp"))
-	{
-		mozilla_register_FtpProtocolHandler ();
-	}
-	else
-	{
-		mozilla_unregister_FtpProtocolHandler ();
-	}
-#endif
-
-	mozilla_register_MailtoProtocolHandler ();
-}
-
 static gboolean
 mozilla_init_cookie_observer(void)
 {
@@ -414,6 +289,7 @@ mozilla_init_cookie_observer(void)
 	return TRUE;
 }
 
+#if HAVE_NSICHROMEREGISTRYSEA_H || HAVE_NSIXULCHROMEREGISTRY_SELECTLOCALE
 static nsresult
 getUILang(nsAString& aUILang)
 {
@@ -427,13 +303,7 @@ getUILang(nsAString& aUILang)
 		return NS_ERROR_FAILURE;
 	}
         
-#if MOZILLA_SNAPSHOT > 12
 	result = localeService->GetLocaleComponentForUserAgent (aUILang);
-#else
-	nsXPIDLString uiLang;
-	result = localeService->GetLocaleComponentForUserAgent(getter_Copies(uiLang));
-	aUILang.Assign(uiLang);
-#endif
 	if (NS_FAILED (result))
 	{
 		g_warning ("Could not determine locale!\n");
@@ -449,12 +319,17 @@ static gboolean
 mozilla_init_chrome (void)
 {
 	nsresult rv = NS_OK;
-	nsAutoString uiLang;
+	GulString uiLang;
 	// compute lang code
 	rv = getUILang(uiLang);
 	if (NS_FAILED (rv)) return FALSE;
 
+#ifdef HAVE_NSICHROMEREGISTRYSEA_H
+	nsCOMPtr<nsIChromeRegistrySea> chromeRegistry = do_GetService(NS_CHROMEREGISTRY_CONTRACTID, &rv);
+#else
+
 	nsCOMPtr<nsIXULChromeRegistry> chromeRegistry = do_GetService(NS_CHROMEREGISTRY_CONTRACTID, &rv);
+#endif
 	if (NS_FAILED (rv) || !chromeRegistry)
 	{
 		g_warning ("Could not get the chrome registry!\n");
@@ -466,7 +341,7 @@ mozilla_init_chrome (void)
 
 	// Dont worry if this fails, it is probably because the lang pack
 	// isn't installed.
-	chromeRegistry->SelectLocale (NS_ConvertUCS2toUTF8(uiLang), PR_FALSE);
+	chromeRegistry->SelectLocale (GulCString (uiLang), PR_FALSE);
 
 	// Make sure skin is classic so we get native scrollbars
 	rv = chromeRegistry->SelectSkin (NS_LITERAL_CSTRING("classic/1.0"), PR_FALSE);
@@ -474,15 +349,17 @@ mozilla_init_chrome (void)
 
 	return TRUE;
 }
+#endif
 
 static void
 mozilla_embed_shell_init (MozillaEmbedShell *mes)
 {
-        mes->priv = g_new0 (MozillaEmbedShellPrivate, 1);
+        mes->priv = MOZILLA_EMBED_SHELL_GET_PRIVATE (mes);
 
 	mes->priv->encodings = NULL;
 
 	/* Pre initialization */
+	mozilla_init_plugin_path();
 	mozilla_init_home ();
 	mozilla_init_profile ();
 	
@@ -498,8 +375,10 @@ impl_initialize (GaleonEmbedShell *ges)
 	MozillaEmbedShell *mes = MOZILLA_EMBED_SHELL(ges);
 
 	/* Until gtkmozembed does this itself */
+#if HAVE_NSICHROMEREGISTRYSEA_H || HAVE_NSIXULCHROMEREGISTRY_SELECTLOCALE
 	ret = mozilla_init_chrome();
 	if (ret == FALSE) return FALSE;
+#endif
 
 	/* Post initialization */
 	ret = mozilla_set_default_prefs ();
@@ -512,11 +391,7 @@ impl_initialize (GaleonEmbedShell *ges)
 	ret = mozilla_register_components ();
 	if (ret == FALSE) return FALSE;
 
-	mozilla_init_js_console (mes);
-
-	mozilla_register_external_protocols ();
-
-	mozilla_init_sidebar ();
+	mozilla_init_js_console ();
 
 	mozilla_init_cookie_observer();
 		
@@ -561,7 +436,7 @@ mozilla_embed_shell_new_window_orphan_cb (GtkMozEmbedSingle *embed,
                 }
         }
        
-	g_signal_emit_by_name (shell, "new_window_orphan", &new_embed, mask);
+	g_signal_emit_by_name (shell, "new_window_orphan", mask, &new_embed);
 
 	g_assert (new_embed != NULL);
 	
@@ -576,7 +451,7 @@ mozilla_embed_shell_finalize (GObject *object)
 	MozillaEmbedShell *mes;
 
         g_return_if_fail (object != NULL);
-        g_return_if_fail (IS_MOZILLA_EMBED_SHELL (object));
+        g_return_if_fail (MOZILLA_IS_EMBED_SHELL (object));
 
         mes = MOZILLA_EMBED_SHELL (object);
 
@@ -592,111 +467,46 @@ mozilla_embed_shell_finalize (GObject *object)
 	{
 		g_object_unref (mes->priv->encodings);
 	}
-        g_free (mes->priv);
 
-        G_OBJECT_CLASS (parent_class)->finalize (object);
+        G_OBJECT_CLASS (mozilla_embed_shell_parent_class)->finalize (object);
 }
 
-static void              
-impl_get_capabilities (GaleonEmbedShell *shell,
-		       EmbedShellCapabilities *caps)
-{
-	EmbedShellCapabilities mycaps;
-	
-	mycaps = (EmbedShellCapabilities)
-	 	 (CACHE_CLEAR_CAP |
-	          OFFLINE_CAP |
-        	  PROXY_AUTOCONF_CAP |
-	          JAVA_CONSOLE_CAP |
-	          JS_CONSOLE_CAP |
-	          CHARSETS_CAP |
-	          PERMISSIONS_CAP |
-	          COOKIES_CAP |
-	          PASSWORDS_CAP);
-	
-	*caps = mycaps;
-}
-
-static gresult      
+static void
 impl_clear_cache (GaleonEmbedShell *shell,
 		  CacheType type)
 {
-	nsresult rv;
-	
 	nsCOMPtr<nsICacheService> CacheService =
-                        do_GetService (NS_CACHESERVICE_CONTRACTID, &rv);
-	if (NS_FAILED(rv)) return G_FAILED;
+                        do_GetService (NS_CACHESERVICE_CONTRACTID);
 
-	CacheService->EvictEntries((guint)type);
-
-	return G_OK;
+	if (CacheService)
+	{
+		CacheService->EvictEntries((guint)type);
+	}
 }
 
-static gresult          
+static void
 impl_set_offline_mode (GaleonEmbedShell *shell,
 		       gboolean offline)
 {
-	nsresult rv;
-
-	nsCOMPtr<nsIIOService> io = do_GetService(NS_IOSERVICE_CONTRACTID, &rv);
-        if (NS_FAILED(rv))
-                return G_FAILED;
-
-        rv = io->SetOffline(offline);
-	if (NS_SUCCEEDED(rv)) return G_FAILED;
-	
-	return G_OK;
+	nsCOMPtr<nsIIOService> io = do_GetService(NS_IOSERVICE_CONTRACTID);
+	if (io)
+	{
+		io->SetOffline(offline);
+	}
 }
 
-static gresult           
-impl_load_proxy_autoconf (GaleonEmbedShell *shell,
-			  const char* url)
-{
-	nsresult rv;
-
-        nsCOMPtr<nsIProtocolProxyService> pps =
-                do_GetService ("@mozilla.org/network/protocol-proxy-service;1",
-                               &rv);
-        if (NS_FAILED(rv) || !pps) return G_FAILED;
-
-        rv = pps->ConfigureFromPAC (url);
-	if (NS_FAILED(rv)) return G_FAILED;
-	
-	return G_OK;
-}
-
-static gresult           
+static void
 impl_show_java_console (GaleonEmbedShell *shell)
 {
-	nsresult rv;
-        nsCOMPtr<nsIJVMManager> JVMManager = do_GetService (kJVMManagerCID,
-                                                            &rv);
-        if (NS_FAILED(rv)) return G_FAILED;
-
-        rv = JVMManager->ShowJavaConsole ();
-	if (NS_FAILED(rv)) return G_FAILED;
-	
-	return G_OK;
+        nsCOMPtr<nsIJVMManager> JVMManager = do_GetService (kJVMManagerCID);
+	if (JVMManager)
+	{
+		JVMManager->ShowJavaConsole ();
+	}
 }
 
-static gresult           
-impl_shell_show_js_console (GaleonEmbedShell *shell)
-{
-	nsresult rv;
-        nsCOMPtr<nsIJSConsoleService> JSConsoleService =
-                do_GetService ("@mozilla.org/embedcomp/jsconsole-service;1",
-                               &rv);
-        if (NS_FAILED(rv)) return G_FAILED;
-
-        rv = JSConsoleService->Open (nsnull);
-	if (NS_FAILED(rv)) return G_FAILED;
-
-	return G_OK;
-}
-
-static gresult           
-impl_get_encodings (GaleonEmbedShell *shell,
-		    GaleonEncodings **encodings)
+static GaleonEncodings *
+impl_get_encodings (GaleonEmbedShell *shell)
 {
 	MozillaEmbedShell *mshell = MOZILLA_EMBED_SHELL(shell);
 
@@ -704,14 +514,12 @@ impl_get_encodings (GaleonEmbedShell *shell,
 	{
 		mshell->priv->encodings = mozilla_encodings_new ();
 	}
-	*encodings = GALEON_ENCODINGS (mshell->priv->encodings);
-	return G_OK;
+	return GALEON_ENCODINGS (mshell->priv->encodings);
 }
 
 
-static gresult
-impl_get_font_langs (GaleonEmbedShell *shell,
-		     GList **langs)
+static GList *
+impl_get_font_langs (GaleonEmbedShell *shell)
 {
 	GList *l = NULL;
 	int i;
@@ -722,57 +530,43 @@ impl_get_font_langs (GaleonEmbedShell *shell,
 		info->title = _(font_languages[i].title);
 		l = g_list_append (l, info);
 	}
-	*langs = l;
-	return G_OK;
+	return l;
 }
 
 
-static gresult
+static GList *
 impl_get_font_list (GaleonEmbedShell *shell,
 		    const char *langGroup,
-		    const char *fontType,
-		    GList **fontList,
-		    char **default_font)
+		    const char *fontType)
 {
 	nsresult rv;
 
 	nsCOMPtr<nsIFontEnumerator> mozFontEnumerator;
 	mozFontEnumerator = do_CreateInstance("@mozilla.org/gfx/fontenumerator;1", &rv);
-	NS_ENSURE_SUCCESS(rv, G_FAILED);
+	NS_ENSURE_SUCCESS(rv, NULL);
 
 	PRUint32    n_fonts;
 	PRUnichar **fonts;
 	rv = mozFontEnumerator->EnumerateFonts (langGroup, fontType,
 			                        &n_fonts, &fonts);
-	NS_ENSURE_SUCCESS(rv, G_FAILED);
+	NS_ENSURE_SUCCESS(rv, NULL);
 
 	GList *l = NULL;
 	for (PRUint32 i = 0; i < n_fonts; i++)
 	{
 		char *str;
 
-		str = g_strdup (NS_ConvertUCS2toUTF8(fonts[i]).get());
+		str = g_strdup (GulCString (fonts[i]).get());
 		l = g_list_prepend (l, str);
 		nsMemory::Free (fonts[i]);
 	}
 
 	nsMemory::Free (fonts);
 
-	*fontList = g_list_reverse (l);
-
-	if (default_font != NULL)
-	{
-		char key [255];
-
-		sprintf (key, "font.name.%s.%s", fontType, langGroup);
-		
-		*default_font = mozilla_prefs_get_string (key);
-	}
-
-	return G_OK;
+	return g_list_reverse (l);
 }
 
-static gresult           
+static void
 impl_set_permission(GaleonEmbedShell *shell,
 		    const char *url, 
 		    PermissionType type,
@@ -783,36 +577,27 @@ impl_set_permission(GaleonEmbedShell *shell,
                         do_GetService(NS_PERMISSIONMANAGER_CONTRACTID);
 
 	nsCOMPtr<nsIURI> uri;
-	rv = NS_NewURI(getter_AddRefs(uri), url);
-	if (NS_FAILED(rv) || !uri) return G_FAILED;
+	rv = GaleonUtils::NewURI(getter_AddRefs(uri), GulDependentCString (url));
+	if (NS_FAILED(rv) || !uri) return;
 
-	#if MOZILLA_SNAPSHOT > 9
-	rv = permissionManager->Add(uri, gPermType[type],
-	#else
-	rv = permissionManager->Add(uri, type,
-	#endif
-				    allow ? (PRUint32)nsIPermissionManager::ALLOW_ACTION
-				    	  : (PRUint32)nsIPermissionManager::DENY_ACTION);
-
-	if (NS_FAILED(rv)) return G_FAILED;
-
-	return G_OK;
+	 permissionManager->Add(uri, gPermType[type],
+				allow ? (PRUint32)nsIPermissionManager::ALLOW_ACTION
+				: (PRUint32)nsIPermissionManager::DENY_ACTION);
 }
 
-static gresult           
+static GList *
 impl_list_permissions (GaleonEmbedShell *shell,
-		       PermissionType type, 
-		       GList **permissions)
+		       PermissionType type)
 {
         nsresult result;
 
-	*permissions = NULL;
+	GList *permissions = NULL;
 	
 	nsCOMPtr<nsIPermissionManager> permissionManager = 
                         do_GetService(NS_PERMISSIONMANAGER_CONTRACTID);
         nsCOMPtr<nsISimpleEnumerator> permissionEnumerator;
         result = permissionManager->GetEnumerator(getter_AddRefs(permissionEnumerator));
-        if (NS_FAILED(result)) return G_FAILED;
+        if (NS_FAILED(result)) return NULL;
 	
         PRBool enumResult;
         for (permissionEnumerator->HasMoreElements(&enumResult) ;
@@ -821,20 +606,14 @@ impl_list_permissions (GaleonEmbedShell *shell,
         {
                 nsCOMPtr<nsIPermission> nsPermission;
                 result = permissionEnumerator->GetNext(getter_AddRefs(nsPermission));
-                if (NS_FAILED(result)) return G_FAILED;
+                if (NS_FAILED(result)) return NULL;
 
-#if MOZILLA_SNAPSHOT > 9
-		nsCAutoString cType;
+		GulCString cType;
 		nsPermission->GetType(cType);
 		if (cType.Equals(gPermType[type]))
-#else
-		PRUint32 cType;
-                nsPermission->GetType(&cType);
-                if ((PRUint32)cType == (PRUint32)type)
-#endif
                 {
                         PermissionInfo *b = g_new0(PermissionInfo, 1);
-			nsCString host;
+			GulCString host;
 			nsPermission->GetHost(host);
 			b->domain = g_strdup(host.get());
 
@@ -851,16 +630,14 @@ impl_list_permissions (GaleonEmbedShell *shell,
 			  default :
 			  	b->status = FALSE;
 			}
-                        *permissions = g_list_prepend(*permissions, b);
+                        permissions = g_list_prepend(permissions, b);
                 }
         }
 
-        *permissions = g_list_reverse(*permissions);
-	
-	return G_OK;
+        return g_list_reverse(permissions);
 }
 
-static gresult           
+static void
 impl_remove_permissions(GaleonEmbedShell *shell,
 		        PermissionType type,
 		        GList *permissions)
@@ -872,61 +649,44 @@ impl_remove_permissions(GaleonEmbedShell *shell,
         for (GList *l = permissions; l != NULL; l = l->next)
         {
                 PermissionInfo *b = (PermissionInfo *)l->data;
-                result = permissionManager->Remove(nsDependentCString(b->domain),
-#if MOZILLA_SNAPSHOT > 9
+                result = permissionManager->Remove(GulDependentCString (b->domain),
 						   gPermType[type]);
-#else
-                				   type);
-#endif
-                if (NS_FAILED(result)) return G_FAILED;
+                if (NS_FAILED(result)) return;
         };
-
-	return G_OK;
 }
 
-static gresult           
+static PermissionActionType
 impl_test_permission (GaleonEmbedShell *shell,
 		      const char *url, 
-		      PermissionType type,
-		      PermissionActionType *ret_action)
+		      PermissionType type)
 {
 	nsresult rv;
 	nsCOMPtr<nsIPermissionManager> permissionManager =
                         do_GetService(NS_PERMISSIONMANAGER_CONTRACTID);
 
-	if (!url) return G_FAILED;
+	if (!url) return DENY_ACTION;
 
 	nsCOMPtr<nsIURI> uri;
-	rv = NS_NewURI(getter_AddRefs(uri), url);
-	if (NS_FAILED(rv) || !uri) return G_FAILED;
+	rv = GaleonUtils::NewURI(getter_AddRefs(uri), GulDependentCString (url));
+	if (NS_FAILED(rv) || !uri) return DENY_ACTION;
 
 	PRUint32 action;
-#if MOZILLA_SNAPSHOT > 9
 	rv = permissionManager->TestPermission(uri, gPermType[type], &action);
-#else
-	rv = permissionManager->TestPermission(uri, type, &action);
-#endif
 
 	switch (action)
 	{
 	  case nsIPermissionManager::UNKNOWN_ACTION :
-	  	*ret_action = UNKNOWN_ACTION;
-	  	break;
+		  return UNKNOWN_ACTION;
 	  case nsIPermissionManager::ALLOW_ACTION :
-	  	*ret_action = ALLOW_ACTION;
-	  	break;
+		  return ALLOW_ACTION;
 	  case nsIPermissionManager::DENY_ACTION :
-	  	*ret_action = DENY_ACTION;
-	  	break;
+		  return DENY_ACTION;
 	}
-	if (NS_FAILED(rv)) return G_FAILED;
-
-	return G_OK;
+	return DENY_ACTION;
 }
 
-static gresult           
-impl_list_cookies (GaleonEmbedShell *shell,
-		   GList **cookies)
+static GList *
+impl_list_cookies (GaleonEmbedShell *shell)
 {
         nsresult result;
         nsCOMPtr<nsICookieManager> cookieManager = 
@@ -934,8 +694,9 @@ impl_list_cookies (GaleonEmbedShell *shell,
         nsCOMPtr<nsISimpleEnumerator> cookieEnumerator;
         result = 
             cookieManager->GetEnumerator(getter_AddRefs(cookieEnumerator));
-        if (NS_FAILED(result)) return G_FAILED;
+        if (NS_FAILED(result)) return NULL;
 
+	GList *cookies = NULL;
         PRBool enumResult;
         for (cookieEnumerator->HasMoreElements(&enumResult) ;
              enumResult == PR_TRUE ;
@@ -943,18 +704,16 @@ impl_list_cookies (GaleonEmbedShell *shell,
         {
                 nsCOMPtr<nsICookie> nsCookie;
                 result = cookieEnumerator->GetNext(getter_AddRefs(nsCookie));
-                if (NS_FAILED(result)) return G_FAILED;
+                if (NS_FAILED(result)) return NULL;
 
                 CookieInfo *c = mozilla_cookie_to_info(nsCookie);
-                *cookies = g_list_prepend(*cookies, c);
+                cookies = g_list_prepend(cookies, c);
         }       
 
-	*cookies = g_list_reverse(*cookies);
-		
-	return G_OK;
+	return g_list_reverse(cookies);
 }
 
-static gresult           
+static void
 impl_remove_cookies (GaleonEmbedShell *shell,
 		     GList *cookies,
 		     gboolean block)
@@ -967,20 +726,19 @@ impl_remove_cookies (GaleonEmbedShell *shell,
         {
                 CookieInfo *c = (CookieInfo *)l->data;
 
-                result = cookieManager->Remove (nsDependentCString(c->domain),
-                                                nsDependentCString(c->name),
-                                                nsDependentCString(c->path),
+                result = cookieManager->Remove (GulDependentCString(c->domain),
+                                                GulDependentCString(c->name),
+                                                GulDependentCString(c->path),
                                                 block);
-                if (NS_FAILED(result)) return G_FAILED;
+                if (NS_FAILED(result)) return;
         };
 
-	return G_OK;
+	return;
 }
 	
-static gresult           
+static GList *
 impl_list_passwords (GaleonEmbedShell *shell,
-		     PasswordType type, 
-		     GList **passwords)
+		     PasswordType type)
 {
         nsresult result = NS_ERROR_FAILURE;
 
@@ -993,7 +751,9 @@ impl_list_passwords (GaleonEmbedShell *shell,
         else if (type == PASSWORD_REJECT)
                 result = passwordManager->GetRejectEnumerator 
                                 (getter_AddRefs(passwordEnumerator));
-        if (NS_FAILED(result)) return G_FAILED;      
+        if (NS_FAILED(result)) return NULL;
+
+	GList *passwords = NULL;
 
         PRBool enumResult;
         for (passwordEnumerator->HasMoreElements(&enumResult) ;
@@ -1003,30 +763,28 @@ impl_list_passwords (GaleonEmbedShell *shell,
                 nsCOMPtr<nsIPassword> nsPassword;
                 result = passwordEnumerator->GetNext 
                                         (getter_AddRefs(nsPassword));
-                if (NS_FAILED(result)) return G_FAILED;
+                if (NS_FAILED(result)) return NULL;
 
                 PasswordInfo *p = g_new0 (PasswordInfo, 1);
 
-                nsCAutoString transfer;
+                GulCString transfer;
                 nsPassword->GetHost (transfer);
                 p->host = g_strdup (transfer.get());
 
                 if (type == PASSWORD_PASSWORD)
                 {
-                        nsAutoString unicodeName;
+                        GulString unicodeName;
                         nsPassword->GetUser (unicodeName);
-                        p->username = g_strdup(NS_ConvertUCS2toUTF8(unicodeName).get());
+                        p->username = g_strdup (GulCString (unicodeName).get());
                 }
 
-		*passwords = g_list_prepend (*passwords, p);
+		passwords = g_list_prepend (passwords, p);
         }       
 
-	*passwords = g_list_reverse (*passwords);
-
-	return G_OK;
+	return g_list_reverse (passwords);
 }
 
-static gresult           
+static void
 impl_remove_passwords (GaleonEmbedShell *shell,
 		       GList *passwords, 
 		       PasswordType type)
@@ -1040,18 +798,63 @@ impl_remove_passwords (GaleonEmbedShell *shell,
                 PasswordInfo *p = (PasswordInfo *)l->data;
                 if (type == PASSWORD_PASSWORD)
                 {
-                        result = passwordManager->RemoveUser (nsDependentCString(p->host),
-                                                              NS_ConvertUTF8toUCS2(nsDependentCString(p->username)));
+                        result = passwordManager->RemoveUser (GulDependentCString(p->host),
+							      GulString (p->username));
                 }
                 else if (type == PASSWORD_REJECT)
                 {
-                        result = passwordManager->RemoveReject
-                                        (nsDependentCString(p->host));
+                        result = passwordManager->RemoveReject 
+				(GulDependentCString(p->host));
                 };
 
-                if (NS_FAILED(result)) return G_FAILED;
+                if (NS_FAILED(result)) return;
         };
-	
-        return G_OK;
 }
 
+static GList *
+impl_get_printer_info(GaleonEmbedShell *shell,
+                      gint *defaultPrinterIndex)
+{
+	return MozillaPrivate::GetPrinterList (defaultPrinterIndex);
+}
+
+
+static void
+mozilla_embed_shell_class_init (MozillaEmbedShellClass *klass)
+{
+	GObjectClass *object_class = G_OBJECT_CLASS (klass);
+	GaleonEmbedShellClass *shell_class;
+	
+	shell_class = GALEON_EMBED_SHELL_CLASS (klass);
+	
+        object_class->finalize = mozilla_embed_shell_finalize;
+
+	shell_class->initialize = impl_initialize;
+	shell_class->clear_cache = impl_clear_cache;
+	shell_class->set_offline_mode = impl_set_offline_mode;
+	shell_class->show_java_console = impl_show_java_console;
+	shell_class->get_encodings   = impl_get_encodings;
+	shell_class->get_font_langs = impl_get_font_langs;
+	shell_class->get_font_list = impl_get_font_list;
+	shell_class->set_permission = impl_set_permission;
+	shell_class->list_permissions = impl_list_permissions;
+	shell_class->remove_permissions = impl_remove_permissions;
+	shell_class->test_permission = impl_test_permission;
+	shell_class->list_cookies = impl_list_cookies;
+	shell_class->remove_cookies = impl_remove_cookies;
+	shell_class->list_passwords = impl_list_passwords;
+	shell_class->remove_passwords = impl_remove_passwords;
+	shell_class->get_printer_info = impl_get_printer_info;
+
+	g_type_class_add_private (klass, sizeof (MozillaEmbedShellPrivate));
+}
+
+GaleonEmbedShell  *
+galeon_embed_shell_new()
+{
+	GaleonEmbedShell *shell;
+	
+	shell = GALEON_EMBED_SHELL (g_object_new (MOZILLA_TYPE_EMBED_SHELL, NULL));
+
+	return shell;
+}
