@@ -34,6 +34,8 @@
 #include "hig-alert.h"
 #include "migrate.h"
 #include "galeon-debug.h"
+#include "gul-x11.h"
+#include "gul-gui.h"
 
 #include <libgnomeui/gnome-client.h>
 #include <bonobo/bonobo-main.h>
@@ -41,6 +43,7 @@
 #include <gtk/gtksignal.h>
 #include <gtk/gtkmain.h>
 #include <gtk/gtkmessagedialog.h>
+#include <gtk/gtkversion.h>
 #include <gconf/gconf-client.h>
 
 #ifdef ENABLE_NAUTILUS_VIEW
@@ -52,6 +55,10 @@
 
 #endif
 
+#define GALEON_SHELL_GET_PRIVATE(object) (G_TYPE_INSTANCE_GET_PRIVATE ((object), \
+				       GALEON_TYPE_SHELL, GaleonShellPrivate))
+
+
 struct GaleonShellPrivate
 {
 	GbBookmarkSet *bookmark_set;
@@ -63,6 +70,7 @@ struct GaleonShellPrivate
 	EggToolbarsModel *toolbars_model;
 	gboolean server_mode;
 	BonoboGenericFactory *automation_factory;
+	GaleonSidebars * sidebars;
 };
 
 enum
@@ -135,11 +143,12 @@ galeon_shell_class_init (GaleonShellClass *klass)
         parent_class = g_type_class_peek_parent (klass);
 
         object_class->finalize = galeon_shell_finalize;
+
+	g_type_class_add_private (klass, sizeof (GaleonShellPrivate));
 }
 
-static void
+static GaleonEmbed *
 galeon_shell_new_window_cb (GaleonEmbedShell *shell, 
-			    GaleonEmbed **new_embed,
                             EmbedChromeMask chromemask,
 			    gpointer data)
 {
@@ -147,8 +156,6 @@ galeon_shell_new_window_cb (GaleonEmbedShell *shell,
         GaleonWindow *window;
 	gboolean open_in_tab;
 
-	g_assert (new_embed != NULL);
-
 	open_in_tab = (chromemask & EMBED_CHROME_OPENASCHROME) ?
 		      FALSE :
 		      eel_gconf_get_boolean (CONF_TABS_TABBED_POPUPS);
@@ -166,14 +173,41 @@ galeon_shell_new_window_cb (GaleonEmbedShell *shell,
 	new_tab = galeon_tab_new ();
 	galeon_window_add_tab (window, new_tab,
 			       GALEON_WINDOW_ADD_TAB_LAST, FALSE);
-	*new_embed = galeon_tab_get_embed (new_tab);
+	
+	return galeon_tab_get_embed (new_tab);
 }
 
+
+/*
+ * Evaluate the command in the top embed of the window that the
+ * jsconsole is transient for. If it isn't transient for any window,
+ * then use the active window.
+ */
+static void
+galeon_shell_evaluate_cb (GaleonJSConsole *jsconsole, const char *cmd, GaleonShell *gs)
+{
+	GtkWindow *window;
+	GaleonEmbed *embed;
+
+	window = gtk_window_get_transient_for (GTK_WINDOW (jsconsole));
+	if (!window)
+	{
+		window = GTK_WINDOW (galeon_shell_get_active_window (gs));
+	}
+
+	embed = galeon_window_get_active_embed (GALEON_WINDOW (window));
+	g_return_if_fail (GALEON_IS_EMBED (embed));
+
+	galeon_embed_evaluate_javascript (embed, cmd);
+}
+
+
 static void
 galeon_shell_init (GaleonShell *gs)
 {
 	gboolean ret;
 	GConfSchema *schema;
+	GaleonJSConsole *jsconsole;
 
 	galeon_shell = gs;
 	g_object_add_weak_pointer (G_OBJECT(galeon_shell),
@@ -181,7 +215,7 @@ galeon_shell_init (GaleonShell *gs)
 	
 	gul_pixbuf_cache_register_stocks ();
 	
-        gs->priv = g_new0 (GaleonShellPrivate, 1);
+        gs->priv = GALEON_SHELL_GET_PRIVATE (gs);
 	gs->priv->session     = NULL;
 	gs->priv->server_mode = FALSE;
 
@@ -234,11 +268,7 @@ galeon_shell_init (GaleonShell *gs)
 	gconf_schema_free (schema);
 
 
-#ifdef ENABLE_GTKHTML_EMBED
-	gs->priv->embed_shell = galeon_embed_shell_new ("gtkhtml");	
-#elif ENABLE_MOZILLA_EMBED
-	gs->priv->embed_shell = galeon_embed_shell_new ("mozilla");	
-#endif
+	gs->priv->embed_shell = galeon_embed_shell_new ();
 	g_assert (gs->priv->embed_shell != NULL);
 
 	/* Now that we have all the important stuff created, migrate any
@@ -274,6 +304,12 @@ galeon_shell_init (GaleonShell *gs)
 			  G_CALLBACK(galeon_shell_new_window_cb),
 			  NULL);
 	
+	jsconsole = galeon_embed_shell_get_js_console (embed_shell);
+	g_signal_connect (jsconsole,
+			  "evaluate",
+			  G_CALLBACK(galeon_shell_evaluate_cb),
+			  galeon_shell);
+
 	gs->priv->favicon_cache = galeon_favicon_cache_new ();
 
 	galeon_init_services (gs);
@@ -285,7 +321,7 @@ galeon_shell_finalize (GObject *object)
         GaleonShell *gs;
 
         g_return_if_fail (object != NULL);
-        g_return_if_fail (IS_GALEON_SHELL (object));
+        g_return_if_fail (GALEON_IS_SHELL (object));
 
 	gs = GALEON_SHELL (object);
 
@@ -293,7 +329,7 @@ galeon_shell_finalize (GObject *object)
 
 	g_assert (galeon_shell == NULL);
 	
-	g_return_if_fail (IS_GALEON_EMBED_SHELL (gs->priv->embed_shell));
+	g_return_if_fail (GALEON_IS_EMBED_SHELL (gs->priv->embed_shell));
 	g_object_unref (G_OBJECT (gs->priv->embed_shell));
 
 	if (gs->priv->session)
@@ -318,6 +354,11 @@ galeon_shell_finalize (GObject *object)
 		g_object_unref (gs->priv->toolbars_model);
 	}
 
+	if (gs->priv->sidebars)
+	{
+		g_object_unref (gs->priv->sidebars);
+	}
+
 	g_return_if_fail (GALEON_IS_FAVICON_CACHE(gs->priv->favicon_cache));
 	g_object_unref (G_OBJECT (gs->priv->favicon_cache));
 
@@ -334,7 +375,6 @@ galeon_shell_finalize (GObject *object)
 		bonobo_object_unref (gs->priv->automation_factory);
 	}
 	
-	g_free (gs->priv);
 
 	bonobo_main_quit ();
 
@@ -344,7 +384,7 @@ galeon_shell_finalize (GObject *object)
 GaleonShell *
 galeon_shell_new (void)
 {
-	return GALEON_SHELL (g_object_new (GALEON_SHELL_TYPE, NULL));
+	return GALEON_SHELL (g_object_new (GALEON_TYPE_SHELL, NULL));
 }
 
 /**
@@ -358,7 +398,7 @@ galeon_shell_new (void)
 GaleonEmbedShell *
 galeon_shell_get_embed_shell (GaleonShell *gs)
 {
-	g_return_val_if_fail (IS_GALEON_SHELL (gs), NULL);
+	g_return_val_if_fail (GALEON_IS_SHELL (gs), NULL);
 
 	return gs->priv->embed_shell;
 }
@@ -506,6 +546,31 @@ galeon_shell_new_tab (GaleonShell *shell,
 		      GaleonTab *previous_tab,
 		      const char *url,
 		      GaleonNewTabFlags flags)
+{
+	return galeon_shell_new_tab_full (shell, parent_window, 
+					  previous_tab, url, flags, 0u);
+}
+
+/**
+ * galeon_shell_new_tab_full:
+ * @shell: a #GaleonShell
+ * @parent_window: the target #GaleonWindow or %NULL
+ * @previous_tab: the referrer tab or %NULL
+ * @url: an url to load or %NULL
+ * @user_time: The time of the event that caused the window to open
+ *
+ * Create a new tab and the parent window when necessary.
+ * Ever use this function to open urls in new window/tabs.
+ *
+ * ReturnValue: the created #GaleonTab
+ **/
+GaleonTab *
+galeon_shell_new_tab_full (GaleonShell *shell,
+			   GaleonWindow *parent_window,
+			   GaleonTab *previous_tab,
+			   const char *url,
+			   GaleonNewTabFlags flags,
+			   guint32 user_time)
 {
 	GaleonEmbed * previous_embed = NULL;
 
@@ -514,11 +579,10 @@ galeon_shell_new_tab (GaleonShell *shell,
 		previous_embed = galeon_tab_get_embed (previous_tab);
 	}
 	
-	return galeon_shell_new_tab_from_embed
-		(shell, parent_window, previous_embed, url, flags);
+	return galeon_shell_new_tab_from_embed_full
+		(shell, parent_window, previous_embed, url, flags, user_time);
 }
 
-
 /**
  * galeon_shell_new_tab_from_embed:
  * @shell: a #GaleonShell
@@ -537,6 +601,35 @@ galeon_shell_new_tab_from_embed (GaleonShell *shell,
 				 GaleonEmbed *previous_embed,
 				 const char *url,
 				 GaleonNewTabFlags flags)
+{
+	return galeon_shell_new_tab_from_embed_full (shell,
+						     parent_window,
+						     previous_embed,
+						     url,
+						     flags,
+						     0u);
+}
+
+/**
+ * galeon_shell_new_tab_from_embed_full:
+ * @shell: a #GaleonShell
+ * @parent_window: the target #GaleonWindow or %NULL
+ * @previous_embed: the referrer embed or %NULL
+ * @url: an url to load or %NULL
+ * @user_time: The time of the event that caused the window to open
+ *
+ * Create a new tab and the parent window when necessary.
+ * Ever use this function to open urls in new window/tabs.
+ *
+ * ReturnValue: the created #GaleonTab
+ **/
+GaleonTab *
+galeon_shell_new_tab_from_embed_full (GaleonShell *shell,
+				      GaleonWindow *parent_window,
+				      GaleonEmbed *previous_embed,
+				      const char *url,
+				      GaleonNewTabFlags flags,
+				      guint32 user_time)
 {
 	GaleonWindow *window;
 	GaleonTab *tab;
@@ -553,6 +646,7 @@ galeon_shell_new_tab_from_embed (GaleonShell *shell,
 	jump_to = eel_gconf_get_boolean (CONF_TABS_TABBED_AUTOJUMP);
 	
 	if (flags & GALEON_NEW_TAB_JUMP) jump_to = TRUE;
+	if (flags & GALEON_NEW_TAB_RAISE_WINDOW) jump_to = TRUE;
 	if (flags & GALEON_NEW_TAB_DONT_JUMP_TO) jump_to = FALSE;
 
 	if (!in_new_window && parent_window != NULL)
@@ -562,6 +656,7 @@ galeon_shell_new_tab_from_embed (GaleonShell *shell,
 	else
 	{
         	window = galeon_window_new ();
+		in_new_window = TRUE;
 	}
 
 	position = (flags & GALEON_NEW_TAB_APPEND) ?
@@ -574,6 +669,9 @@ galeon_shell_new_tab_from_embed (GaleonShell *shell,
 	galeon_window_add_tab (window, tab,
 			       position,
 			       jump_to);
+
+	gul_x11_window_update_user_time (GTK_WIDGET (window), user_time);
+
 	gtk_widget_show (GTK_WIDGET(window));
 
 	embed = galeon_tab_get_embed (tab);
@@ -581,28 +679,32 @@ galeon_shell_new_tab_from_embed (GaleonShell *shell,
 	{
 		load_homepage (shell, embed, previous_embed);
 	}
-	else if ((flags & GALEON_NEW_TAB_IS_A_COPY) ||
-		 (flags & GALEON_NEW_TAB_VIEW_SOURCE))
+	else if (flags & GALEON_NEW_TAB_VIEW_SOURCE)
 	{
-		EmbedDisplayType display_type =
-			(flags & GALEON_NEW_TAB_VIEW_SOURCE) ?
-			 DISPLAY_AS_SOURCE : DISPLAY_NORMAL;
 		g_return_val_if_fail (previous_embed != NULL, tab);
-		galeon_embed_copy_page (embed, previous_embed, display_type);
+		galeon_embed_copy_page (embed, previous_embed, DISPLAY_AS_SOURCE);
+	}
+	else if (flags & GALEON_NEW_TAB_IS_A_COPY)
+	{
+		g_return_val_if_fail (previous_embed != NULL, tab);
+		galeon_embed_shistory_copy (previous_embed, embed, TRUE, TRUE, TRUE);
 	}
 	else if (url)
 	{
 		galeon_embed_load_url (embed, url);
 	}
 
-	if (embed && previous_embed)
+	if (embed && previous_embed && 
+	    !(flags & GALEON_NEW_TAB_IS_A_COPY) &&
+	    (flags & GALEON_NEW_TAB_COPY_HISTORY))
 	{
 		galeon_embed_shistory_copy (previous_embed, embed, TRUE, FALSE, FALSE);
 	}
 
-	if (flags & GALEON_NEW_TAB_RAISE_WINDOW)
+	/* Only forceably raise the window if it hasn't just been created */
+	if (!in_new_window && (flags & GALEON_NEW_TAB_RAISE_WINDOW))
 	{
-		gtk_window_present (GTK_WINDOW (window));
+		gul_gui_window_present (GTK_WINDOW (window), user_time);
 	}
 
 	if (flags & GALEON_NEW_TAB_FULLSCREEN)
@@ -610,6 +712,26 @@ galeon_shell_new_tab_from_embed (GaleonShell *shell,
 		gtk_window_fullscreen (GTK_WINDOW (window));
 	}
 
+	/* Make sure the initial focus is somewhere sensible and not, for
+	 * example, on the reload button.
+	 */
+	if (in_new_window || jump_to)
+	{
+		/* If the location entry is blank, focus that, except if the
+		 * page was a copy */
+		if (galeon_tab_is_empty (tab) &&
+		    !(flags & (GALEON_NEW_TAB_VIEW_SOURCE|GALEON_NEW_TAB_IS_A_COPY)))
+		{
+			/* empty page, focus location entry */
+			galeon_window_edit_location (window);
+		}
+		else 
+		{
+			/* non-empty page, focus the page */
+			gtk_widget_grab_focus (GTK_WIDGET (embed));
+		}
+	}
+
         return tab;
 }
 
@@ -639,6 +761,7 @@ galeon_nautilus_view_new (BonoboGenericFactory *factory, const char *id,
 static void
 bookmarks_init (GaleonShell *gs)
 {
+	GlobalHistory *gh = galeon_embed_shell_get_global_history (gs->priv->embed_shell);
 	/* this is temp code, I need to think about how to properly initialize bookmarks */
 	GbXBEL *io = gb_xbel_new ();
 	gchar *filename = g_strconcat (g_get_home_dir (), "/",
@@ -703,6 +826,7 @@ bookmarks_init (GaleonShell *gs)
 	}
 
 	gb_bookmark_set_set_auto_save (gs->priv->bookmark_set, 5 * 60 * 1000);
+	gb_bookmark_set_set_autobookmarks_source (gs->priv->bookmark_set, GALEON_AUTO_BOOKMARKS_SOURCE (gh));
 
 	gb_galeon_icon_provider_add_bookmark_set (gip, gs->priv->bookmark_set);
 
@@ -872,3 +996,13 @@ galeon_shell_get_toolbars_model  (GaleonShell *gs)
 	return gs->priv->toolbars_model;
 }
 
+
+GaleonSidebars *
+galeon_shell_get_sidebars (GaleonShell *gs)
+{
+	if (!gs->priv->sidebars)
+	{
+		gs->priv->sidebars = galeon_sidebars_new (gs->priv->embed_shell);
+	}
+	return gs->priv->sidebars;
+}
