@@ -21,69 +21,56 @@
 
 #include "galeon-embed.h"
 #include "galeon-embed-persist.h"
-#include "galeon-embed-popup-bw.h"
 #include "galeon-sidebar.h"
 #include "galeon-dialog.h"
+#include "gul-notebook.h"
+#include "page-info-dialog.h"
 #include <glib-object.h>
 #include <glib.h>
-#include <bonobo/bonobo-window.h>
+#include <gtk/gtkwindow.h>
 #include "bookmarks-location-source.h"
+#include "bookmarks-util.h"
+#include "egg-editable-toolbar.h"
 
 G_BEGIN_DECLS
 
 typedef struct GaleonWindowClass GaleonWindowClass;
-
-#define GALEON_WINDOW_TYPE             (galeon_window_get_type ())
-#define GALEON_WINDOW(obj)             (GTK_CHECK_CAST ((obj), GALEON_WINDOW_TYPE, GaleonWindow))
-#define GALEON_WINDOW_CLASS(klass)     (GTK_CHECK_CLASS_CAST ((klass), GALEON_WINDOW, GaleonWindowClass))
-#define IS_GALEON_WINDOW(obj)          (GTK_CHECK_TYPE ((obj), GALEON_WINDOW_TYPE))
-#define IS_GALEON_WINDOW_CLASS(klass)  (GTK_CHECK_CLASS_TYPE ((klass), GALEON_WINDOW))
-
 typedef struct GaleonWindow GaleonWindow;
 typedef struct GaleonWindowPrivate GaleonWindowPrivate;
-typedef struct Toolbar Toolbar;
+
+#include "galeon-popup.h"
+
+#define GALEON_TYPE_WINDOW             (galeon_window_get_type ())
+#define GALEON_WINDOW(obj)             (GTK_CHECK_CAST ((obj), GALEON_TYPE_WINDOW, GaleonWindow))
+#define GALEON_WINDOW_CLASS(klass)     (GTK_CHECK_CLASS_CAST ((klass), GALEON_WINDOW, GaleonWindowClass))
+#define GALEON_IS_WINDOW(obj)          (GTK_CHECK_TYPE ((obj), GALEON_TYPE_WINDOW))
+#define GALEON_IS_WINDOW_CLASS(klass)  (GTK_CHECK_CLASS_TYPE ((klass), GALEON_WINDOW))
 
 struct GaleonWindow 
 {
-        BonoboWindow parent;
+	GtkWindow parent;
         GaleonWindowPrivate *priv;
 
-	/* Public to toolbar and statusbar, dont use outside */
-	GObject *ui_component;
+	/* Public for ppv toolbar */
+	GObject *egg_dock;
+
+	/* Public for popup menus, ppv toolbar and maybe status bar */
+	GtkUIManager *merge;
 };
 
 struct GaleonWindowClass
 {
-        BonoboWindowClass parent_class;
+	GtkWindowClass parent_class;
+
+	/* signals */
+	void (*active_embed_changed)	(GaleonWindow *window, GaleonEmbed *old_embed, GaleonEmbed *new_embed);
 };
 
-typedef enum
+enum
 {
-	NormalMode,
-	FullscreenMode
-} GaleonWindowMode;
-
-typedef enum 
-{
-	TabsControl,
-	NavControl,
-	FindControl,
-	ZoomControl,
-	CharsetsControl,
-	TitleControl,
-	LocationControl,
-	FaviconControl,
-	StatusbarSecurityControl,
-	StatusbarMessageControl,
-	StatusbarProgressControl,
-	StatusbarPopupBlockControl,
-	SpinnerControl,
-	WindowVisibilityControl,
-	BMAndHistoryControl,
-	TabsAppeareanceControl,
-	PermissionControl,
-	CssMenuControl,
-} ControlID;
+        GALEON_WINDOW_ADD_TAB_LAST = GUL_NOTEBOOK_INSERT_LAST,
+        GALEON_WINDOW_ADD_TAB_GROUPED = GUL_NOTEBOOK_INSERT_GROUPED
+};
 
 /* Include the header down here to resolve circular dependency */
 #include "galeon-tab.h"
@@ -99,13 +86,23 @@ EmbedChromeMask  galeon_window_get_chrome	  (GaleonWindow *window);
 
 void		 galeon_window_toggle_fullscreen  (GaleonWindow *window);
 
+void		galeon_window_bookmark_activate	  (GaleonWindow *w, 
+						   GbBookmarkEventActivated *ev);
+
+gboolean	 galeon_window_can_close	  (GaleonWindow *w);
+
+void             galeon_window_update_edit_actions_sensitivity (GaleonWindow *w, gboolean hide);
+
+void             galeon_window_enable_edit_actions_sensitivity (GaleonWindow *w);
+
 /* Tabs */
 
 void          	 galeon_window_add_tab      	  (GaleonWindow *window, 
 					     	   GaleonTab *tab,
+						   gint position,
 						   gboolean jump_to);
 
-void          	 galeon_window_remove_tab      	  (GaleonWindow *window, 
+gboolean	 galeon_window_remove_tab	  (GaleonWindow *window, 
 					     	   GaleonTab *tab);
 
 void		 galeon_window_next_tab		  (GaleonWindow *window);
@@ -123,14 +120,6 @@ void		 galeon_window_reparent_tab	  (GaleonWindow *window,
 		                                   GaleonWindow *source,
 						   GaleonTab *tab);
 
-void		 galeon_window_set_tab_status	  (GaleonWindow *window,
-		                                   GaleonTab *tab,
-						   TabLoadStatus status);
-
-void		 galeon_window_set_tab_title	  (GaleonWindow *window,
-		                                   GaleonTab *tab,
-						   const char *title);
-
 void	      	 galeon_window_load_url      	  (GaleonWindow *window,
 					     	   const char *url);
 
@@ -139,10 +128,12 @@ void		 galeon_window_set_zoom		  (GaleonWindow *window,
 
 void             galeon_window_edit_location	  (GaleonWindow *window);
 
-void          	 galeon_window_update_control 	  (GaleonWindow *window,
-					    	   ControlID control);
+gchar * 	 galeon_window_get_location_entry_location (GaleonWindow *window);
 
-void		 galeon_window_update_all_controls (GaleonWindow *window);
+void	 	 galeon_window_set_location_entry_location (GaleonWindow *window, 
+							    const gchar *location);
+
+GtkWidget	 *galeon_window_get_statusbar	  (GaleonWindow *window);
 
 GaleonSidebar	 *galeon_window_get_sidebar	  (GaleonWindow *window);
 
@@ -150,23 +141,25 @@ GaleonTab    	 *galeon_window_get_active_tab 	  (GaleonWindow *window);
 
 GaleonEmbed  	 *galeon_window_get_active_embed  (GaleonWindow *window);
 
-GaleonEmbedPopupBW *galeon_window_get_popup_factory (GaleonWindow *window);
+GaleonPopup	 *galeon_window_get_popup_factory (GaleonWindow *window);
+
+GulNotebook	 *galeon_window_get_notebook	  (GaleonWindow *window);
 
 GList 	 	 *galeon_window_get_tabs	  (GaleonWindow *window);
 
-Toolbar		 *galeon_window_get_toolbar	  (GaleonWindow *window);
-
-void		  galeon_window_site_visited	  (GaleonWindow *window, 
-						   const gchar *url, const gchar *title);
+EggEditableToolbar *galeon_window_get_toolbar	  (GaleonWindow *window);
 
 void              galeon_window_clear_history     (GaleonWindow *window);
 
 /* Dialogs */
+void		  galeon_window_show_page_info	(GaleonWindow *window, PageInfoDialogPage page);
 
-GaleonDialog     *galeon_window_get_find_dialog   (GaleonWindow *window);
+void		  galeon_window_show_find	 (GaleonWindow *window);
 
 void		  galeon_window_show_history	  (GaleonWindow *window);
 
+void              galeon_window_show_js_console   (GaleonWindow *window);
+
 G_END_DECLS
 
 #endif
