@@ -30,10 +30,10 @@ G_BEGIN_DECLS
 typedef struct PPViewToolbar PPViewToolbar;
 typedef struct PPViewToolbarClass PPViewToolbarClass;
 
-#define PPVIEW_TOOLBAR_TYPE             (ppview_toolbar_get_type ())
-#define PPVIEW_TOOLBAR(obj)             (GTK_CHECK_CAST ((obj), PPVIEW_TOOLBAR_TYPE, PPViewToolbar))
+#define TYPE_PPVIEW_TOOLBAR             (ppview_toolbar_get_type ())
+#define PPVIEW_TOOLBAR(obj)             (GTK_CHECK_CAST ((obj), TYPE_PPVIEW_TOOLBAR, PPViewToolbar))
 #define PPVIEW_TOOLBAR_CLASS(klass)     (GTK_CHECK_CLASS_CAST ((klass), PPVIEW_TOOLBAR, PPViewToolbarClass))
-#define IS_PPVIEW_TOOLBAR(obj)          (GTK_CHECK_TYPE ((obj), PPVIEW_TOOLBAR_TYPE))
+#define IS_PPVIEW_TOOLBAR(obj)          (GTK_CHECK_TYPE ((obj), TYPE_PPVIEW_TOOLBAR))
 #define IS_PPVIEW_TOOLBAR_CLASS(klass)  (GTK_CHECK_CLASS_TYPE ((klass), PPVIEW_TOOLBAR))
 
 typedef struct PPViewToolbarPrivate PPViewToolbarPrivate;
